from __future__ import print_function

from builtins import range
import os
import random
import time
from argparse import ArgumentParser

from couchapp.commands import push as couchapppush

from WMCore.Database.CMSCouch import CouchServer
from WMCore.Lexicon import splitCouchServiceURL
from WMCore.WMBase import getWMBASE

NUM_OF_REQUEST = 20
ITERATIONS = 100
NUM_OF_JOBS_PER_REQUEST = 10


def couchAppRoot():
    """Return path to couchapp dir"""
    wmBase = getWMBASE()
    develPath = "%s/src/couchapps" % wmBase
    if not os.path.exists(develPath):
        basePath = "%s/couchapps" % os.environ['WMCORE_ROOT']
    else:
        basePath = develPath
    return basePath


def installCouchApp(couchUrl, couchDBName, couchAppName, basePath=None):
    """
    _installCouchApp_

    Install the given couch app on the given server in the given database.  If
    the database already exists it will be deleted.
    """
    if not basePath:
        basePath = couchAppRoot()
    print("Installing %s into %s" % (couchAppName, couchDBName))

    couchapppush("%s/%s" % (basePath, couchAppName), "%s/%s" % (couchUrl, couchDBName))
    return


def parse_opts():
    parser = ArgumentParser()
    parser.add_argument("-d", "--dburl",
                        dest="dburl",
                        help="CouchDB URL which data will be populated")
    parser.add_argument("-c", "--couchapp-base",
                        dest="couchapp_base",
                        help="Couch sapp base path")
    parser.add_argument("--no-couchapp",
                        action="store_false",
                        default=True,
                        dest="add_couchapp",
                        help="Don't update couchapp")
    parser.add_argument("--no-reqmgr-data",
                        action="store_false",
                        default=True,
                        dest="add_reqmgr_data",
                        help="Don't update reqmgr data")
    parser.add_argument("--no-agent-data",
                        action="store_false",
                        default=True,
                        dest="add_agent_data",
                        help="Don't update reqmgr data")
    parser.add_argument("-u", "--users",
                        dest="users",
                        default=10,
                        type=int,
                        help="The number of users, default=10")
    parser.add_argument("-s", "--sites",
                        dest="sites",
                        default=5,
                        type=int,
                        help="The number of sites, default=5")
    parser.add_argument("-a", "--agents",
                        dest="agents",
                        default=2,
                        type=int,
                        help="The number of agents, default=2")
    parser.add_argument("-i", "--iterations",
                        dest="iterations",
                        default=ITERATIONS,
                        type=int,
                        help="The number of iterations to make, default=%s" % ITERATIONS)
    parser.add_argument("-r", "--requests",
                        dest="requests",
                        default=NUM_OF_REQUEST,
                        type=int,
                        help="The number of requests to simulate, default=%s" % NUM_OF_REQUEST)
    parser.add_argument("-w", "--wait",
                        dest="wait",
                        default=0,
                        type=int,
                        help="Wait W seconds between iterations, default=0")

    return parser.parse_args()


def generate_reqmgr_requests(number=NUM_OF_REQUEST):
    """
    generate the request with following structure
      {
       "_id": "cmsdataops_sryu_test4_120111_114950_128",
       "_rev": "6-02b17b4eabcf333e7499c0fa0ae5055b",
       "inputdataset": "/Photon/Run2011A-v1/RAW",
       "group": "cmsdataops",
       "request_date": [2012, 1, 11, 17, 49, 50],
       "campaign": "SryuTest2",
       "workflow": "cmsdataops_sryu_test4_120111_114950_128",
       "priority": "1",
       "requestor": "cmsdataops",
       "request_type": "ReReco",
       "type": "reqmgr_request",
       "request_status": [
           {
               "status": "new",
               "update_time": 1326304190
           },
           {
               "status": "assignment-approved",
               "update_time": 1326304216
           },
           {
               "status": "assigned",
               "update_time": 1326304227
           }
       ],
       "site_white_list": [
           "T1_DE_KIT"
       ],
       "team": "cmsdataops"
    }
    """
    docs = []
    for i in range(number):
        doc = {"_id": "test_workflow_%s" % i,
               "inputdataset": "/Photon/Run2011A-v1/RAW",
               "group": "cmsdataops",
               "request_date": [2012, 1, 11, 17, 49, 50],
               "campaign": "SryuTest-%s" % (i % 5),
               "workflow": "test_workflow_%s" % i,
               "priority": "1",
               "requestor": "cmsdataops",
               "request_type": "ReReco",
               "type": "reqmgr_request",
               "request_status": [
                   {"status": "new", "update_time": 1326304190},
                   {"status": "assignment-approved", "update_time": 1326304216},
                   {"status": "assigned", "update_time": 1326304227}
               ],
               "site_white_list": ["T1_DE_KIT"],
               "team": "cmsdataops"
               }
        docs.append(doc)
    return docs


def generate_agent_requests(number=NUM_OF_REQUEST, iterations=ITERATIONS):
    """
    generate the request with following structure
      {
       "_id": "af27057919546ff8f3fc8d7f18233355",
       "_rev": "1-181021c38a5444676d7718f42ffa9a89",
       "status": {
           "inWMBS": 1,
           "queued": {
               "retry": 1
           }
       },
       "workflow": "cmsdataops_sryu_test4_120111_114950_128",
       "timestamp": 1326306397,
       "sites": {
           "T1_DE_KIT": {
               "queued": {
                   "retry": 1
               }
           }
       },
       "agent": "WMAgent",
       "team": "team1,team2,cmsdataops",
       "agent_url": "cms-xen39.fnal.gov",
       "type": "agent_request"
       }
    """
    current_time = int(time.time())
    docs = []
    for cycle in range(iterations):
        for i in range(number):
            doc = {"status": {"inWMBS": 12,
                              "submitted": {"retry": 2, "running": 2, "pending": 2, "first": 2},
                              "failure": {"exception": 2, "create": 2, "submit": 2},
                              "queued": {"retry": 2, "first": 2},
                              "canceled": 2,
                              "cooloff": 2,
                              "success": 2
                              },

                   "workflow": "test_workflow_%s" % i,
                   "timestamp": current_time + (cycle * 10),
                   "sites": {"T1_DE_KIT":
                       {
                           "submitted": {"retry": 1, "running": 1, "pending": 1, "first": 1},
                           "failure": {"exception": 1, "create": 1, "submit": 1},
                           "queued": {"retry": 1, "first": 1},
                           "canceled": 1,
                           "cooloff": 1,
                           "success": 1
                       },
                       "T1_US_FNAL":
                           {
                               "submitted": {"retry": 1, "running": 1, "pending": 1, "first": 1},
                               "failure": {"exception": 1, "create": 1, "submit": 1},
                               "queued": {"retry": 1, "first": 1},
                               "canceled": 1,
                               "cooloff": 1,
                               "success": 1
                           }
                   },
                   "agent": "WMAgent",
                   "agent_teams": "cmsdataops",
                   "agent_url": "cms-xen39.fnal.gov",
                   "type": "agent_request"
                   }
            docs.append(doc)
    return docs


def generate_jobsummary(request, number=NUM_OF_JOBS_PER_REQUEST):
    """
    jobSummary = {"_id": "jobid_1",  //jobid
                  "type": "jobsummary", // setvalue
                  "retrycount": job["retry_count"],
                  "workflow": workflow1, //request name
                  "task": job["task"],
                  "state": success,
                  "site": T1_US_FNAL,
                  "exitcode": 123,
                  "errors": errmsgs,
                  "lumis": inputs,
                  "output": outputs }

    errmsgs = {}
    inputs = []
    for step in fwjrDocument["fwjr"]["steps"]:
        if "errors" in fwjrDocument["fwjr"]["steps"][step]:
            errmsgs[step] = [error for error in fwjrDocument["fwjr"]["steps"][step]["errors"]]
        if "input" in fwjrDocument["fwjr"]["steps"][step] and "source" in fwjrDocument["fwjr"]["steps"][step]["input"]:
            inputs.extend( [source["runs"] for source in fwjrDocument["fwjr"]['steps'][step]["input"]["source"] if "runs" in source] )
    outputs = [ {'type': singlefile.get('module_label', None),
                 'lfn': singlefile.get('lfn', None),
                 'location': singlefile.get('locations', None),
                 'checksums': singlefile.get('checksums', {}),
                     'size': singlefile.get('size', None) } for singlefile in job["fwjr"].getAllFiles() if singlefile ]


    job status
    ['new', 'created', 'executing', 'complete', 'createfailed', 'submitfailed',
     'jobfailed', 'createcooloff',  'submitcooloff', 'jobcooloff', 'success',
     'exhausted', 'killed']
    """

    # TODO: Make more realistic
    docs = []
    statusList = ['new', 'created', 'executing', 'complete', 'createfailed', 'submitfailed',
                  'jobfailed', 'createcooloff', 'submitcooloff', 'jobcooloff', 'success',
                  'exhausted', 'killed']

    for i in range(number):
        status = statusList[random.randint(0, len(statusList) - 1)]
        errmsgs = {}
        if status.find("failed"):
            exitCode = 666
            errmsgs["step1"] = {}
            errmsgs["step1"]["out"] = {}
            errmsgs["step1"]["out"]["type"] = "test error"
        else:
            exitCode = 0

        jobSummary = {"_id": "jobid_%s_%s" % (request, i),
                      "type": "jobsummary",
                      "retrycount": random.randint(0, 5),
                      "workflow": request,
                      "task": "/%s/task_%s" % (request, i),
                      "state": status,
                      "site": "T1_US_FNAL",
                      "exitcode": exitCode,
                      "errors": errmsgs,
                      "lumis": [[123, 124], [567, 879]],
                      "output": [{'type': "test-type",
                                  'lfn': "/somewhere/file.root",
                                  'location': ['T1_US_FNAL'],
                                  'checksums': {'adler32': 'abc123', 'cksum': 'cdf123'},
                                  'size': "1000"}]
                      }
        docs.append(jobSummary)
    return docs


# def generate_sites(request):
#
#    sites = [ 'T2_AT_Vienna', 'T2_BE_IIHE', 'T2_BE_UCL', 'T2_BR_SPRACE',
#              'T2_BR_UERJ', 'T2_CH_CAF', 'T2_CH_CSCS', 'T2_CN_Beijing', 'T2_DE_DESY',
#              'T2_DE_RWTH', 'T2_EE_Estonia', 'T2_ES_CIEMAT', 'T2_ES_IFCA',
#              'T2_FI_HIP', 'T2_FR_CCIN2P3', 'T2_FR_GRIF_IRFU', 'T2_FR_GRIF_LLR',
#              'T2_FR_IPHC', 'T2_HU_Budapest', 'T2_IN_TIFR', 'T2_IT_Bari',
#              'T2_IT_Legnaro', 'T2_IT_Pisa', 'T2_IT_Rome', 'T2_KR_KNU', 'T2_PK_NCP',
#              'T2_PL_Cracow', 'T2_PL_Warsaw', 'T2_PT_LIP_Lisbon', 'T2_PT_NCG_Lisbon',
#              'T2_RU_IHEP', 'T2_RU_INR', 'T2_RU_ITEP', 'T2_RU_JINR', 'T2_RU_PNPI',
#              'T2_RU_RRC_KI', 'T2_RU_SINP', 'T2_TR_METU', 'T2_TW_Taiwan',
#              'T2_UA_KIPT', 'T2_UK_London_Brunel', 'T2_UK_London_IC',
#              'T2_UK_SGrid_Bristol', 'T2_UK_SGrid_RALPP', 'T2_US_Caltech',
#              'T2_US_Florida', 'T2_US_MIT', 'T2_US_Nebraska', 'T2_US_Purdue',
#              'T2_US_UCSD', 'T2_US_Wisconsin']
#    if sites not in request.keys():
#      request["sites"] = {}
#      # jobs run at 1-10 sites
#      req_sites = random.sample(sites, random.randint(1, 10))
#      # can't use a defaultdict because it doesn't thunk
#      for site in req_sites:
#        request["sites"][site] = {}
#
#    status = {}
#    status.update(request['status'])
#
#    for site in request["sites"]:
#      for k, v in status.items():
#        j = random.randint(0, v)
#        request["sites"][site][k] = j
#        status[k] -= j
#
#    # Mop up - must be a better way to do this...
#    site = request["sites"].keys()[-1]
#    for k, v in status.items():
#      request["sites"][site][k] += v
#
# def start_clock(iterations):
#    difference = iterations * datetime.timedelta(minutes=15)
#    weeks, days = divmod(difference.days, 7)
#    minutes, seconds = divmod(difference.seconds, 60)
#    hours, minutes = divmod(minutes, 60)
#
#    print "Running %s iterations " % iterations
#    print "Equivalent to running for %s weeks, %s days, %s hours, %s minutes" % (weeks, days, hours, minutes)
#
#    now = datetime.datetime.now()
#    dt = datetime.timedelta(minutes=15)
#
#    return now, dt

def main(options):
    url, dbName = splitCouchServiceURL(options.dburl)
    db = CouchServer(url).connectDatabase(dbName)
    reqmgr_requests = generate_reqmgr_requests(options.requests)
    agent_requests = generate_agent_requests(options.requests, options.iterations)

    if options.add_couchapp:
        installCouchApp(url, dbName, "WMStats", options.couchapp_base)

    if options.add_reqmgr_data:
        for req in reqmgr_requests:
            db.queue(req)
        db.commit()
        print("Added %s reqmgr requests" % len(reqmgr_requests))

    if options.add_agent_data:
        for req in agent_requests:
            db.queue(req)
            jobDocs = generate_jobsummary(req['workflow'])
            for job in jobDocs:
                db.queue(job)
        db.commit()
        print("Added %s agent requests" % len(agent_requests))
        print("Added %s job Docs" % (len(agent_requests) * len(jobDocs)))


if __name__ == "__main__":
    main(parse_opts())



#!/usr/bin/env python
"""
_ResultSet_t_

Unit tests for the Database/ResultSet class

"""


#Written to test the ResultSet class initially by mnorman
#Dependent on DBCore, specifically DBCore.processData()

from builtins import str, range

import unittest
import threading

from WMCore.WMFactory import WMFactory
from WMCore.Database.ResultSet import ResultSet
from WMQuality.TestInit import TestInit


class ResultSetTest(unittest.TestCase):


    def setUp(self):
        """
        Set up for initializing the ResultSet test class

        """

        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.testInit.setSchema(customModules = ["WMQuality.TestDB"],
                                   useDefault = False)

        self.myThread = threading.currentThread()

        return

    def tearDown(self):
        """
        Delete the ResultSet test class

        """
        self.testInit.clearDatabase()
        return



    def testFullResultSet(self):
        """
        Function to test all the functionality of the ResultSet

        """

        #ResultSet is a very simple function.  This function should
        #create a ResultSet, fill it with data, and then read it
        #out, all in one go.


        testSet = ResultSet()

        testDict = {'column1': 'value1', 'column2':'value2'}
        binds    = [ {'column1':'value3a', 'column2': 'value2a'},
                     {'column1':'value3b', 'column2': 'value2b'},
                     {'column1':'value3c', 'column2': 'value2c'}]


        self.myThread.dbi.processData('insert into test_tablec (column1, column2) values (:column1, :column2)', testDict)
        self.myThread.dbi.processData('insert into test_tablec (column1, column2) values (:column1, :column2)', binds)
        testProxy = self.myThread.dbi.connection().execute('select * from test_tablec')

        #import pdb
        #pdb.set_trace()
        testSet.add(testProxy)

        #Test to make sure fetchone and fetchall both work
        self.assertEqual(str(testSet.fetchone()[0]), 'value1')
        self.assertEqual(str(testSet.fetchall()[-1][1]), 'value2c')


    def test1000Binds(self):

        testSet2 = ResultSet()

        #Now insert and select a huge number of values
        #This depends on the times variable
        binds = [ ]
        largeInsert  = [ ]
        times = 1000

        #For now I don't want to get too dependent on processData() with many binds, since right now
        #it doesn't work.  That does make things awkward though.
        for i in range(times):
            binds = {'column1': 'value1'+str(i), 'column2':'value2'+str(i)}
            self.myThread.dbi.processData("insert into test_tablec (column1, column2) values (:column1, :column2)", binds)



        sql = "select column1 from test_tablec"
        testProxy = self.myThread.dbi.connection().execute(sql)

        testSet2.add(testProxy)

        self.assertEqual(len(testSet2.fetchall()), times)
        dialect = self.myThread.dialect.lower()
        if dialect == 'mysql':
            self.assertEqual(str(testSet2.fetchall()[1][0]), 'value11')
            self.assertEqual(testSet2.fetchone()[0], 'value10')
        elif dialect == 'oracle':
            self.assertEqual(str(testSet2.fetchall()[1][0]), 'value11')
            self.assertEqual(testSet2.fetchone()[0], 'value10')


        return



if __name__ == "__main__":
    unittest.main()



"""
Meta data information API.

"""

import WMCore

from WMCore.REST.Server import RESTEntity, restcall
from WMCore.REST.Tools import tools
from WMCore.REST.Format import JSONFormat


class ServerInfo(RESTEntity):
    def validate(self, apiobj, method, api, param, safe):
        """

        """
        pass

    @restcall(formats = [('application/json', JSONFormat())])
    @tools.expires(secs=-1)
    def get(self):
        """
        """
        wmstats_version = WMCore.__version__
        return "version: %s" % wmstats_version



#!/usr/bin/env python
# encoding: utf-8
"""
CMSSWTemplate_t.py

Created by Dave Evans on 2010-03-30.
Copyright (c) 2010 Fermilab. All rights reserved.
"""

import unittest

from Utils.PythonVersion import PY3
from WMCore.WMSpec.Steps.Templates.CMSSW import CMSSW as CMSSWTemplate
from WMCore.WMSpec.WMStep import makeWMStep
from WMCore.WMSpec.WMWorkload import newWorkload


class CMSSWTemplateTest(unittest.TestCase):
    """
    Unittest for CMSSW Template and Helper

    Builds the step from scratch, applies and checks the template
    Tests the helper methods

    """

    def setUp(self):
        if PY3:
            self.assertItemsEqual = self.assertCountEqual

    def testA(self):
        """
        instantiate & apply template
        """
        workload = newWorkload("UnitTests")
        task = workload.newTask("CMSSWTemplate")
        stepHelper = step = task.makeStep("TemplateTest")
        step = stepHelper.data

        # first up, create the template
        try:
            template = CMSSWTemplate()
        except Exception as ex:
            msg = "Failed to instantiate the CMSSW Step Template"
            msg += str(ex)
            self.fail(msg)

        # now apply it to the step
        try:
            template(step)
        except Exception as ex:
            msg = "Failed to apply template to step"
            msg += str(ex)
            self.fail(msg)

        # TODO: Check the step has the appropriate attributes expected for CMSSW
        self.assertTrue(
                hasattr(step, "application"))
        self.assertTrue(
                hasattr(step.application, "setup"))

    def testB(self):
        """

        try using the helper API to set and get information in the step

        """
        workload = newWorkload("UnitTests")
        task = workload.newTask("CMSSWTemplate")
        stepHelper = step = task.makeStep("TemplateTest")
        step = stepHelper.data
        template = CMSSWTemplate()
        template(step)
        try:
            helper = template.helper(step)
        except Exception as ex:
            msg = "Failure to create CMSSW Step Helper"
            msg += str(ex)
            self.fail(msg)

        helper.cmsswSetup("CMSSW_X_Y_Z", scramArch="slc5_ia32_gcc443")
        helper.addOutputModule(
                "outputModule1", primaryDataset="Primary",
                processedDataset='Processed',
                dataTier='Tier',
                lfnBase="/store/unmerged/whatever"
        )

    def testMulticoreSettings(self):
        """
        test multicore related methods
        """
        workload = newWorkload("UnitTests")
        task = workload.newTask("CMSSWTemplate")
        stepHelper = step = task.makeStep("TemplateTest")
        step = stepHelper.data
        template = CMSSWTemplate()
        template(step)

        helper = template.helper(step)

        self.assertEqual(helper.getNumberOfCores(), 1)
        self.assertEqual(helper.getEventStreams(), 0)
        helper.setNumberOfCores(8)
        self.assertEqual(helper.getNumberOfCores(), 8)
        self.assertEqual(helper.getEventStreams(), 0)
        helper.setNumberOfCores(8, 6)
        self.assertEqual(helper.getNumberOfCores(), 8)
        self.assertEqual(helper.getEventStreams(), 6)

    def testChainedProcessing(self):
        """

        check the chained processing set up is correct

        """
        workload = newWorkload("UnitTests")
        task = workload.newTask("CMSSWTemplate")
        step = task.makeStep("TemplateTest")
        template = CMSSWTemplate()
        helper = template.helper(step.data)
        inputStepName = "some_inputStepName"
        inputOutputModule = "some_inputOutputModule"
        helper.setupChainedProcessing(inputStepName, inputOutputModule)

        self.assertEqual(helper.data.input.chainedProcessing, True)
        self.assertEqual(helper.data.input.inputStepName, "some_inputStepName")
        self.assertEqual(helper.data.input.inputOutputModule, "some_inputOutputModule")

    def testFileProperties(self):
        """
        _testFileProperties_

        Test some CMSSW step output file properties
        """
        step = makeWMStep("cmsRun1")
        step.setStepType("CMSSW")
        template = CMSSWTemplate()
        template(step.data)
        helper = step.getTypeHelper()

        # default values
        self.assertIsNone(helper.getAcqEra(), None)
        self.assertIsNone(helper.getProcStr(), None)
        self.assertIsNone(helper.getProcVer(), None)
        self.assertIsNone(helper.getPrepId(), None)
        self.assertEqual(helper.listOutputModules(), [])

        # now write something to the step object
        helper.setAcqEra("TestAcqEra")
        helper.setProcStr("TestProcStr")
        helper.setProcVer(111)
        helper.setPrepId("TestPrepId")
        helper.addOutputModule("Merged", primaryDataset="Primary",
                               processedDataset="Processed", dataTier="RECO")

        self.assertEqual(helper.getAcqEra(), "TestAcqEra")
        self.assertEqual(helper.getProcStr(), "TestProcStr")
        self.assertEqual(helper.getProcVer(), 111)
        self.assertEqual(helper.getPrepId(), "TestPrepId")
        self.assertItemsEqual(helper.listOutputModules(), ["Merged"])

    def testGPUSettings(self):
        """
        Test GPU methods at CMSSW template level
        """
        workload = newWorkload("UnitTests")
        task = workload.newTask("CMSSWTemplate")
        stepHelper = task.makeStep("TemplateTest")
        step = stepHelper.data
        template = CMSSWTemplate()
        template(step)

        helper = template.helper(step)

        self.assertEqual(helper.getGPURequired(), "forbidden")
        self.assertIsNone(helper.getGPURequirements())
        helper.setGPUSettings("optional", "test 1 2 3")
        self.assertEqual(helper.getGPURequired(), "optional")
        self.assertItemsEqual(helper.getGPURequirements(), "test 1 2 3")
        helper.setGPUSettings("required", {"key1": "value1", "key2": "value2"})
        self.assertEqual(helper.getGPURequired(), "required")
        self.assertItemsEqual(helper.getGPURequirements(), {"key1": "value1", "key2": "value2"})


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
_FileBased_t_

File based splitting test.
"""


import unittest
import os
import threading
import logging
import time

from WMCore.WMBS.File         import File
from WMCore.WMBS.Fileset      import Fileset
from WMCore.WMBS.Job          import Job
from WMCore.WMBS.Subscription import Subscription
from WMCore.WMBS.Workflow     import Workflow
from WMCore.DataStructs.Run   import Run

from WMCore.DAOFactory                   import DAOFactory
from WMCore.JobSplitting.SplitterFactory import SplitterFactory
from WMCore.Services.UUIDLib                import makeUUID
from WMQuality.TestInit                  import TestInit
#from nose.plugins.attrib import attr

class FileBasedTest(unittest.TestCase):
    """
    _FileBasedTest_

    Test file based job splitting.
    """

    def setUp(self):
        """
        _setUp_

        Create two subscriptions: One that contains a single file and one that
        contains multiple files.
        """
        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.testInit.setSchema(customModules = ["WMCore.WMBS"],
                                useDefault = False)

        myThread = threading.currentThread()
        daofactory = DAOFactory(package = "WMCore.WMBS",
                                logger = myThread.logger,
                                dbinterface = myThread.dbi)

        self.nSites = 2
        locationAction = daofactory(classname = "Locations.New")
        for site in range(self.nSites):
            locationAction.execute(siteName = "site%i" % site,
                                   pnn = "T2_CH_CERN_%i" % site)

        return

    def tearDown(self):
        """
        _tearDown_

        Clear out WMBS.
        """
        self.testInit.clearDatabase(modules = ["WMCore.WMBS"])

        return


    def createTestSubscription(self, nFiles, nSites = 1, closeFileset = False):
        """
        _createTestSubscription_

        Create a set of test subscriptions for testing purposes.
        """

        if nSites > self.nSites:
            nSites = self.nSites

        testFileset = Fileset(name = "TestFileset")
        testFileset.create()

        # Create a testWorkflow
        testWorkflow = Workflow(spec = "spec.xml", owner = "Steve",
                                name = "wf001", task="Test")
        testWorkflow.create()

        # Create the files for each site
        for s in range(nSites):
            for i in range(nFiles):
                newFile = File(makeUUID(), size = 1024, events = 100,
                               locations = set(["T2_CH_CERN_%i" % s]))
                newFile.create()
                testFileset.addFile(newFile)
        testFileset.commit()

        testSubscription = Subscription(fileset = testFileset,
                                        workflow = testWorkflow,
                                        split_algo = "MinFileBased",
                                        type = "Processing")
        testSubscription.create()

        # Close the fileset
        if closeFileset:
            testFileset.markOpen(isOpen = False)

        return testSubscription

    def testA_ExactFiles(self):
        """
        _testExactFiles_

        Test file based job splitting when the number of files per job is
        exactly the same as the number of files in the input fileset.
        """
        nFiles = 5
        sub    = self.createTestSubscription(nFiles = nFiles)
        splitter = SplitterFactory()
        jobFactory = splitter(package = "WMCore.WMBS",
                              subscription = sub)

        jobGroups = jobFactory(files_per_job = nFiles)


        self.assertEqual(len(jobGroups), 1)
        self.assertEqual(len(jobGroups[0].jobs), 1)
        self.assertEqual(len(jobGroups[0].jobs[0]['input_files']), nFiles)
        return

    def testB_LessFilesOpen(self):
        """
        _LessFilesOpen_

        Test with less files then required.
        If the fileset is open, this should produce no jobs.
        """

        sub    = self.createTestSubscription(nFiles = 5)
        splitter = SplitterFactory()
        jobFactory = splitter(package = "WMCore.WMBS",
                              subscription = sub)

        jobGroups = jobFactory(files_per_job = 10)

        self.assertEqual(len(jobGroups), 0)
        return

    def testC_LessFilesClosed(self):
        """
        _LessFilesClosed_

        Test with less files then required.
        If the fileset is closed, this should produce one job.
        """

        sub    = self.createTestSubscription(nFiles = 5, closeFileset = True)
        splitter = SplitterFactory()
        jobFactory = splitter(package = "WMCore.WMBS",
                              subscription = sub)

        jobGroups = jobFactory(files_per_job = 10)

        self.assertEqual(len(jobGroups), 1)
        self.assertEqual(len(jobGroups[0].jobs), 1)
        self.assertEqual(len(jobGroups[0].jobs[0]['input_files']), 5)
        return

    def testD_MoreFilesOpen(self):
        """
        _MoreFilesOpen_

        If you pass it more files then files_per_job, it should produce
        jobs until it hits the limit, then stop.
        """

        sub        = self.createTestSubscription(nFiles = 10)
        splitter   = SplitterFactory()
        jobFactory = splitter(package = "WMCore.WMBS",
                              subscription = sub)

        jobGroups = jobFactory(files_per_job = 3)

        self.assertEqual(len(jobGroups), 1)
        self.assertEqual(len(jobGroups[0].jobs), 3)
        for job in jobGroups[0].jobs:
            self.assertEqual(len(job['input_files']), 3)
        return

    def testE_MoreFilesClosed(self):
        """
        _MoreFilesClosed_

        If you pass it more files then files_per_job, it should produce
        jobs enough to hold all the files in the fileset if the
        fileset is closed
        """

        sub        = self.createTestSubscription(nFiles = 10, closeFileset = True)
        splitter   = SplitterFactory()
        jobFactory = splitter(package = "WMCore.WMBS",
                              subscription = sub)

        jobGroups = jobFactory(files_per_job = 3)

        self.assertEqual(len(jobGroups), 1)
        self.assertEqual(len(jobGroups[0].jobs), 4)
        for job in jobGroups[0].jobs[:3]:
            self.assertEqual(len(job['input_files']), 3)
        self.assertEqual(len(jobGroups[0].jobs[3]['input_files']), 1)
        return


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
Unit tests for the RucioInjectorPoller module
"""

from __future__ import division

import threading
import unittest

from Utils.PythonVersion import PY3

from WMComponent.DBS3Buffer.DBSBufferBlock import DBSBufferBlock
from WMComponent.DBS3Buffer.DBSBufferFile import DBSBufferFile
from WMComponent.RucioInjector.RucioInjectorPoller import RucioInjectorPoller, RucioInjectorException
from WMCore.DAOFactory import DAOFactory
from WMCore.DataStructs.Run import Run
from WMCore.Services.Rucio.Rucio import WMRucioException
from WMCore.Services.UUIDLib import makeUUID
from WMQuality.Emulators.EmulatedUnitTestCase import EmulatedUnitTestCase
from WMQuality.TestInit import TestInit


class RucioInjectorPollerTest(EmulatedUnitTestCase):
    """
    Tests for the RucioInjectorPoller component
    """

    def setUp(self):
        """
        Install the DBSBuffer schema into the database and connect to Rucio.
        """
        super(RucioInjectorPollerTest, self).setUp()
        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection(destroyAllDatabase=True)

        self.testInit.setSchema(customModules=["WMComponent.DBS3Buffer"],
                                useDefault=False)

        myThread = threading.currentThread()
        daofactory = DAOFactory(package="WMComponent.DBS3Buffer",
                                logger=myThread.logger,
                                dbinterface=myThread.dbi)

        locationAction = daofactory(classname="DBSBufferFiles.AddLocation")
        self.locations = ["T2_CH_CERN", "T1_US_FNAL_Disk"]
        for rse in self.locations:
            locationAction.execute(siteName=rse)

        self.testFilesA = []
        self.testFilesB = []
        self.testDatasetA = "/SampleA/PromptReco-v1/RECO"
        self.testDatasetB = "/SampleB/CRUZET11-v1/RAW"

        if PY3:
            self.assertItemsEqual = self.assertCountEqual

        return

    def tearDown(self):
        """
        Delete the database.
        """
        self.testInit.clearDatabase()

    def createConfig(self):
        """
        Create a basic configuration for the component
        """
        config = self.testInit.getConfiguration()
        config.component_("RucioInjector")
        config.RucioInjector.pollInterval = 300
        config.RucioInjector.pollIntervalRules = 43200
        config.RucioInjector.cacheExpiration = 2 * 24 * 60 * 60  # two days
        config.RucioInjector.createBlockRules = True
        config.RucioInjector.RSEPostfix = False  # enable it to append _Test to the RSE names
        config.RucioInjector.metaDIDProject = "Production"
        config.RucioInjector.containerDiskRuleParams = {"weight": "ddm_quota", "copies": 2, "grouping": "DATASET"}
        config.RucioInjector.blockRuleParams = {}
        config.RucioInjector.containerDiskRuleRSEExpr = "(tier=2|tier=1)&cms_type=real&rse_type=DISK"
        config.RucioInjector.rucioAccount = "wma_test"
        config.RucioInjector.rucioUrl = "http://cms-rucio-int.cern.ch"
        config.RucioInjector.rucioAuthUrl = "https://cms-rucio-auth-int.cern.ch"
        return config

    def stuffDatabase(self):
        """
        Fill the dbsbuffer tables with some files and blocks.  We'll insert a total
        of 5 files spanning two blocks.  There will be a total of two datasets
        inserted into the database.
        We'll inject files with the location set as an SE name as well as a
        PhEDEx node name as well.
        """
        myThread = threading.currentThread()

        # Create the DAOs factory and the relevant instances
        buffer3Factory = DAOFactory(package="WMComponent.DBS3Buffer",
                                    logger=myThread.logger,
                                    dbinterface=myThread.dbi)
        setBlock = buffer3Factory(classname="DBSBufferFiles.SetBlock")
        fileStatus = buffer3Factory(classname="DBSBufferFiles.SetStatus")
        associateWorkflow = buffer3Factory(classname="DBSBufferFiles.AssociateWorkflowToFile")
        insertWorkflow = buffer3Factory(classname="InsertWorkflow")
        datasetAction = buffer3Factory(classname="NewDataset")
        createAction = buffer3Factory(classname="CreateBlocks")

        # Create workflow in the database
        insertWorkflow.execute("BogusRequest", "BogusTask", 0, 0, 0, 0)

        # First file on first block
        checksums = {"adler32": "1234", "cksum": "5678"}
        testFileA = DBSBufferFile(lfn=makeUUID(), size=1024, events=10,
                                  checksums=checksums,
                                  locations=set(["T2_CH_CERN"]))
        testFileA.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                               appFam="RECO", psetHash="GIBBERISH",
                               configContent="MOREGIBBERISH")
        testFileA.setDatasetPath(self.testDatasetA)
        testFileA.addRun(Run(2, *[45]))
        testFileA.create()

        # Second file on first block
        testFileB = DBSBufferFile(lfn=makeUUID(), size=1024, events=10,
                                  checksums=checksums,
                                  locations=set(["T2_CH_CERN"]))
        testFileB.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                               appFam="RECO", psetHash="GIBBERISH",
                               configContent="MOREGIBBERISH")
        testFileB.setDatasetPath(self.testDatasetA)
        testFileB.addRun(Run(2, *[45]))
        testFileB.create()

        # Third file on first block
        testFileC = DBSBufferFile(lfn=makeUUID(), size=1024, events=10,
                                  checksums=checksums,
                                  locations=set(["T2_CH_CERN"]))
        testFileC.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                               appFam="RECO", psetHash="GIBBERISH",
                               configContent="MOREGIBBERISH")
        testFileC.setDatasetPath(self.testDatasetA)
        testFileC.addRun(Run(2, *[45]))
        testFileC.create()

        self.testFilesA.append(testFileA)
        self.testFilesA.append(testFileB)
        self.testFilesA.append(testFileC)

        # First file on second block
        testFileD = DBSBufferFile(lfn=makeUUID(), size=1024, events=10,
                                  checksums=checksums,
                                  locations=set(["T1_US_FNAL_Disk"]))
        testFileD.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                               appFam="RECO", psetHash="GIBBERISH",
                               configContent="MOREGIBBERISH")
        testFileD.setDatasetPath(self.testDatasetB)
        testFileD.addRun(Run(2, *[45]))
        testFileD.create()

        # Second file on second block
        testFileE = DBSBufferFile(lfn=makeUUID(), size=1024, events=10,
                                  checksums=checksums,
                                  locations=set(["T1_US_FNAL_Disk"]))
        testFileE.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                               appFam="RECO", psetHash="GIBBERISH",
                               configContent="MOREGIBBERISH")
        testFileE.setDatasetPath(self.testDatasetB)
        testFileE.addRun(Run(2, *[45]))
        testFileE.create()

        self.testFilesB.append(testFileD)
        self.testFilesB.append(testFileE)

        # insert datasets in the dbsbuffer table
        datasetAction.execute(datasetPath=self.testDatasetA)
        datasetAction.execute(datasetPath=self.testDatasetB)

        self.blockAName = self.testDatasetA + "#" + makeUUID()
        self.blockBName = self.testDatasetB + "#" + makeUUID()

        # create and insert blocks into dbsbuffer table
        newBlockA = DBSBufferBlock(name=self.blockAName,
                                   location="T2_CH_CERN",
                                   datasetpath=None)
        newBlockA.setDataset(self.testDatasetA, 'data', 'VALID')
        newBlockA.status = 'Closed'

        newBlockB = DBSBufferBlock(name=self.blockBName,
                                   location="T1_US_FNAL_Disk",
                                   datasetpath=None)
        newBlockB.setDataset(self.testDatasetB, 'data', 'VALID')
        newBlockB.status = 'Closed'

        createAction.execute(blocks=[newBlockA, newBlockB])

        # associate files to their correspondent block id
        setBlock.execute(testFileA["lfn"], self.blockAName)
        setBlock.execute(testFileB["lfn"], self.blockAName)
        setBlock.execute(testFileC["lfn"], self.blockAName)
        setBlock.execute(testFileD["lfn"], self.blockBName)
        setBlock.execute(testFileE["lfn"], self.blockBName)

        # set file status to LOCAL
        fileStatus.execute(testFileA["lfn"], "LOCAL")
        fileStatus.execute(testFileB["lfn"], "LOCAL")
        fileStatus.execute(testFileC["lfn"], "LOCAL")
        fileStatus.execute(testFileD["lfn"], "LOCAL")
        fileStatus.execute(testFileE["lfn"], "LOCAL")

        # associate files to a given workflow
        associateWorkflow.execute(testFileA["lfn"], "BogusRequest", "BogusTask")
        associateWorkflow.execute(testFileB["lfn"], "BogusRequest", "BogusTask")
        associateWorkflow.execute(testFileC["lfn"], "BogusRequest", "BogusTask")
        associateWorkflow.execute(testFileD["lfn"], "BogusRequest", "BogusTask")
        associateWorkflow.execute(testFileE["lfn"], "BogusRequest", "BogusTask")

        return

    def testBadConfig(self):
        """
        Test wrong component configuration
        """
        config = self.createConfig()
        config.RucioInjector.metaDIDProject = "Very invalid project name"

        with self.assertRaises(RucioInjectorException):
            RucioInjectorPoller(config)

    def testActivityMap(self):
        """
        Initialize a RucioInjectorPoller object and test `_activityMap` method
        """
        poller = RucioInjectorPoller(self.createConfig())
        # test production agent and non-Tape endpoint
        activity = poller._activityMap("T1_US_FNAL_Disk")
        self.assertEquals(activity, "Production Output")
        activity = poller._activityMap("T1_US_FNAL_Test")
        self.assertEquals(activity, "Production Output")
        # test production agent and Tape endpoint (which is forbidden at the moment)
        with self.assertRaises(WMRucioException):
            poller._activityMap("T1_US_FNAL_Tape")

        # now pretend it to be a T0 agent/component
        poller.isT0agent = True
        # test T0 agent and non-Tape endpoint
        activity = poller._activityMap("T1_US_FNAL_Disk")
        self.assertEquals(activity, "T0 Export")
        activity = poller._activityMap("T1_US_FNAL_Test")
        self.assertEquals(activity, "T0 Export")
        # test T0 agent and Tape endpoint
        activity = poller._activityMap("T1_US_FNAL_Tape")
        self.assertEquals(activity, "T0 Tape")

    def testLoadingFiles(self):
        """
        Initialize a RucioInjectorPoller object and load uninjected files
        """
        self.stuffDatabase()
        poller = RucioInjectorPoller(self.createConfig())
        poller.setup(parameters=None)
        uninjectedFiles = poller.getUninjected.execute()
        self.assertItemsEqual(list(uninjectedFiles), self.locations)
        self.assertEquals(list(uninjectedFiles["T2_CH_CERN"]), [self.testDatasetA])
        self.assertEquals(list(uninjectedFiles["T1_US_FNAL_Disk"]), [self.testDatasetB])


if __name__ == '__main__':
    unittest.main()



#! /usr/bin/env python

"""
Version of Services/CRIC intended to be used with mock or unittest.mock
"""
from __future__ import division, print_function

from builtins import object, str, bytes

import os
import json
import re
from WMCore.WMBase import getTestBase
from RestClient.ErrorHandling.RestClientExceptions import HTTPError

# Read in the data just once so that we don't have to do it for every test (in __init__)
mockData = {}
globalFile = os.path.join(getTestBase(), '..', 'data', 'Mock', 'CRICMockData.json')
print("Reading mocked CRIC data from the file %s" % globalFile)

try:
    with open(globalFile, 'r') as mockFile:
        mockData = json.load(mockFile)
except IOError:
    mockData = {}

class MockCRICApi(object):
    def __init__(self, url=None, logger=None, configDict=None):
        print("Using MockCRICApi")

    def genericLookup(self, callname):
        """
        This function returns the mocked CRIC data

        :param callname: the CRIC REST API name
        :return: the dictionary that CRIC would have returned
        """
        if callname not in mockData:
            raise RuntimeError("Mock CRIC emulator knows nothing about API %s" % callname)

        if mockData[callname] == 'Raises HTTPError':
            raise HTTPError('http:/cric.mock.fail', 400, 'MockCRIC is raising an exception in place of CRIC',
                            'Dummy header', 'Dummy body')
        else:
            return mockData[callname]

    def userNameDn(self, username):
        callname = 'people'
        res = self.genericLookup(callname)

        userdn = ""
        for x in res:
            if x['username'] == username:
                userdn = x['dn']
                break
        return userdn

    def getAllPSNs(self):
        callname = 'site-names'
        sitenames = self.genericLookup(callname)
        cmsnames = [x['alias'] for x in sitenames if x['type'] == 'psn']
        return cmsnames

    def getAllPhEDExNodeNames(self, pattern=None, excludeBuffer=False):
        callname = 'site-names'
        sitenames = self.genericLookup(callname)
        nodeNames = [x['alias'] for x in sitenames if x['type'] == 'phedex']
        if excludeBuffer:
            nodeNames = [x for x in nodeNames if not x.endswith("_Buffer")]
        if pattern and isinstance(pattern, (str, bytes)):
            pattern = re.compile(pattern)
            nodeNames = [x for x in nodeNames if pattern.match(x)]
        return nodeNames

    def PNNstoPSNs(self, pnns):
        callname = 'data-processing'
        mapping = self.genericLookup(callname)
        if isinstance(pnns, (str, bytes)):
            pnns = [pnns]

        psns = set()
        for pnn in pnns:
            psnSet = set()
            for item in mapping:
                if pnn == item['phedex_name']:
                    psnSet.add(item['psn_name'])
            if psnSet:
                psns.update(psnSet)
        return list(psns)

    def PSNstoPNNs(self, psns, allowPNNLess=False):
        callname = 'data-processing'
        mapping = self.genericLookup(callname)
        if isinstance(psns, (str, bytes)):
            psns = [psns]

        pnns = set()
        for psn in psns:
            pnnSet = set()
            for item in mapping:
                if item['psn_name'] == psn:
                    pnnSet.add(item['phedex_name'])
            if pnnSet:
                pnns.update(pnnSet)
            elif allowPNNLess:
                pnns.add(psn)
        return list(pnns)

    def PSNtoPNNMap(self, psnPattern=''):
        if not isinstance(psnPattern, (str, bytes)):
            raise TypeError('psnPattern argument must be of type str or bytes')

        callname = 'data-processing'
        results = self.genericLookup(callname)
        mapping = {}

        psnPattern = re.compile(psnPattern)
        for entry in results:
            if psnPattern.match(entry['psn_name']):
                mapping.setdefault(entry['psn_name'], set()).add(entry['phedex_name'])
        return mapping



#!/usr/bin/env python
"""
_ListSubsAndFilesetsFromWorkflow_

Oracle implementation of Subscriptions.ListSubsAndFilesetsFromWorkflow
"""
from __future__ import division

from WMCore.WMBS.MySQL.Subscriptions.ListSubsAndFilesetsFromWorkflow import ListSubsAndFilesetsFromWorkflow \
    as ListSubsAndFilesetsFromWorkflowMySQL


class ListSubsAndFilesetsFromWorkflow(ListSubsAndFilesetsFromWorkflowMySQL):
    pass



#!/usr/bin/env python
"""
Some generic base classes for building web pages with.
"""

from builtins import str, bytes

import hashlib
import json
import logging
import os
import time
from datetime import datetime, timedelta
from time import mktime
from wsgiref.handlers import format_date_time

import cherrypy
from Cheetah import Version
from Cheetah.Template import Template
from cherrypy import log as cplog
from cherrypy import request

from Utils.PythonVersion import PY3
from Utils.Utilities import encodeUnicodeToBytesConditional
from WMCore.DataStructs.WMObject import WMObject
from WMCore.WMFactory import WMFactory
from WMCore.Wrappers.JsonWrapper.JSONThunker import JSONThunker

DEFAULT_EXPIRE = 5 * 60


class Page(WMObject):
    """
    __Page__

    Page is a base class that holds a configuration and provides a logger.
    """

    def warning(self, msg=None):
        """
        Log a warning
        """
        self.log(msg, logging.WARNING)

    def exception(self, msg=None):
        """
        Log an exception
        """
        self.log(msg, logging.ERROR)

    def error(self, msg=None):
        """
        Log an error
        """
        self.log(msg, logging.ERROR)

    def debug(self, msg=None):
        """
        Log a debug statement
        """
        self.log(msg, logging.DEBUG)

    def info(self, msg=None):
        """
        Log some info
        """
        self.log(msg, logging.INFO)

    def log(self, msg=None, severity=logging.INFO):
        """
        Do the logging using the CherryPy logger
        """

        if not isinstance(msg, str):
            msg = str(msg)
        if msg:
            cplog.error_log.log(severity, msg)


class TemplatedPage(Page):
    """
    __TemplatedPage__

    TemplatedPage is a class that provides simple Cheetah templating
    """

    def __init__(self, config={}):
        """
        Configure the Page base class then add in the location of the templates. If
        this is not specified in the configuration take a guess based on the location
        of the file.
        """
        Page.__init__(self, config)
        self.templatedir = ''
        if hasattr(self.config, 'templates'):
            self.templatedir = self.config.templates
        else:
            # Take a guess
            self.warning("Configuration doesn't specify template location, guessing %s" % self.templatedir)
            self.templatedir = '%s/%s' % (__file__.rsplit('/', 1)[0], 'Templates')

        self.debug("Templates are located in: %s" % self.templatedir)
        self.debug("Using Cheetah version: %s" % Version)

    def templatepage(self, file=None, *args, **kwargs):
        """
        Apply the cheetah template specified by file to the data in arg and kwargs.
        The templates are compiled in memory and results do not get written to the
        filesystem. The templates do not need to be compiled. Templates are located in
        self.templatedir - this is specified in the configuration section 'templates'.

        You should use cgi.escape to escape data going into the template if you are unsure
        of it's provenance.
        """
        searchList = []
        if len(args) > 0:
            searchList.append(args)
        if len(kwargs) > 0:
            searchList.append(kwargs)
        templatefile = "%s/%s.tmpl" % (self.templatedir, file)
        if os.path.exists(templatefile):
            template = Template(file=templatefile, searchList=searchList)
            return template.respond()
        else:
            self.warning("Template %s not found at %s" % (file, self.templatedir))
            return "Template for page not found"


def _setCherryPyHeaders(data, contentType, expires):
    """
    Convenience function to set headers appropriately
    """
    cherrypy.response.headers['Content-Type'] = contentType
    if data:
        cherrypy.response.headers['Content-Length'] = len(data)
    else:
        cherrypy.response.headers['Content-Length'] = 0
    cherrypy.lib.caching.expires(secs=expires, force=True)
    # TODO: find a better way to generate Etag
    cherrypy.response.headers['ETag'] = data.__str__().__hash__()


def exposeatom(func):
    """
    Convenience decorator function to expose atom XML
    """

    def wrapper(self, data, expires, contentType="application/atom+xml"):
        data = func(self, data)
        _setCherryPyHeaders(data, contentType, expires)
        return self.templatepage('Atom', data=data,
                                 config=self.config,
                                 path=request.path_info)

    wrapper.__doc__ = func.__doc__
    wrapper.__name__ = func.__name__
    wrapper.exposed = True
    return wrapper


def exposexml(func):
    """
    Convenience decorator function to expose XML
    """

    def wrapper(self, data, expires, contentType="application/xml"):
        data = func(self, data)
        _setCherryPyHeaders(data, contentType, expires)
        return self.templatepage('XML', data=data,
                                 config=self.config,
                                 path=request.path_info)

    wrapper.__doc__ = func.__doc__
    wrapper.__name__ = func.__name__
    wrapper.exposed = True
    return wrapper


def exposedasplist(func):
    """
    Convenience decorator function to expose plist XML
    see http://docs.python.org/library/plistlib.html#module-plistlib
    """

    def wrapper(self, data, expires, contentType="application/xml"):
        import plistlib
        data_struct = runDas(self, func, data, expires)
        plist_str = plistlib.writePlistToString(data_struct)
        cherrypy.response.headers['ETag'] = data_struct['results'].__str__().__hash__()
        _setCherryPyHeaders(plist_str, contentType, expires)
        return plist_str

    wrapper.__doc__ = func.__doc__
    wrapper.__name__ = func.__name__
    wrapper.exposed = True
    return wrapper


def exposedasxml(func):
    """
    Convenience decorator function to expose DAS XML

    This will prepend the DAS header to the data and calculate the checksum of
    the data to set the etag correctly

    TODO: pass in the call_time value, can we get this in a smart/neat way?
    TODO: include the request_version in the data hash - a new version should
    result in an update in a cache
    TODO: "inherit" from the exposexml
    """

    def wrapper(self, data, expires, contentType="application/xml"):
        das = runDas(self, func, data, expires)
        header = "<?xml version='1.0' standalone='yes'?>"
        keys = list(das)
        keys.remove('results')
        string = ''
        for key in keys:
            string = '%s %s="%s"' % (string, key, das[key])
        header = "%s\n<das %s>" % (header, string)
        xmldata = header + das['results'].__str__() + "</das>"
        _setCherryPyHeaders(xmldata, contentType, expires)
        return xmldata

    wrapper.__doc__ = func.__doc__
    wrapper.__name__ = func.__name__
    wrapper.exposed = True
    return wrapper


def exposetext(func):
    """
    Convenience decorator function to expose plain text
    """

    def wrapper(self, data, expires, contentType="text/plain"):
        data = func(self, data)
        data = str(data)
        _setCherryPyHeaders(data, contentType, expires)
        return data

    wrapper.__doc__ = func.__doc__
    wrapper.__name__ = func.__name__
    wrapper.exposed = True
    return wrapper


def exposejson(func):
    """
    Convenience decorator function to expose json
    """

    def wrapper(self, data, expires, contentType="application/json"):
        data = func(self, data)
        try:
            #            jsondata = encoder.iterencode(data)
            jsondata = json.dumps(data)
            _setCherryPyHeaders(jsondata, contentType, expires)
            return jsondata
        except Exception:
            raise
            # Exception("Fail to jsontify obj '%s' type '%s'" % (data, type(data)))
        #        return data

    wrapper.__doc__ = func.__doc__
    wrapper.__name__ = func.__name__
    wrapper.exposed = True
    return wrapper


def exposejsonthunker(func):
    """
    Convenience decorator function to expose thunked json
    """

    def wrapper(self, data, expires, contentType="application/json+thunk"):
        data = func(self, data)
        try:
            thunker = JSONThunker()
            data = thunker.thunk(data)
            jsondata = json.dumps(data)
            _setCherryPyHeaders(jsondata, contentType, expires)
            return jsondata
        except Exception:
            raise
            # Exception("Fail to jsontify obj '%s' type '%s'" % (data, type(data)))
        #        return data

    wrapper.__doc__ = func.__doc__
    wrapper.__name__ = func.__name__
    wrapper.exposed = True
    return wrapper


def exposedasjson(func):
    """
    Convenience decorator function to expose DAS json

    This will prepend the DAS header to the data and calculate the checksum of
    the data to set the etag correctly

    TODO: pass in the call_time value, can we get this in a smart/neat way?
    TODO: include the request_version in the data hash - a new version should
    result in an update in a cache
    TODO: "inherit" from the exposejson
    """

    def wrapper(self, data, expires, contentType="application/json"):
        data = runDas(self, func, data, expires)

        try:
            jsondata = json.dumps(data)
            _setCherryPyHeaders(jsondata, contentType, expires)
            return jsondata
        except Exception:
            raise
            # Exception("Failed to json-ify obj '%s' type '%s'" % (data, type(data)))

    wrapper.__doc__ = func.__doc__
    wrapper.__name__ = func.__name__
    wrapper.exposed = True
    return wrapper


def exposejs(func):
    """
    Convenience decorator function to expose javascript
    """

    def wrapper(self, data, expires, contentType="application/javascript"):
        data = func(self, data)
        _setCherryPyHeaders(data, contentType, expires)
        return data

    wrapper.__doc__ = func.__doc__
    wrapper.__name__ = func.__name__
    wrapper.exposed = True
    return wrapper


def exposecss(func):
    """
    Convenience decorator function to expose css
    """

    def wrapper(self, data, expires, contentType="text/css"):
        data = func(self, data)
        _setCherryPyHeaders(data, contentType, expires)
        return data

    wrapper.__doc__ = func.__doc__
    wrapper.__name__ = func.__name__
    wrapper.exposed = True
    return wrapper


def runDas(self, func, data, expires):
    """
    Run a query and produce a dictionary for DAS formatting
    """
    start_time = time.time()
    results = func(self, data)
    call_time = time.time() - start_time
    res_expire = make_timestamp(expires)
    if isinstance(results, list):
        if len(results) > 0:
            row = results[0]
        else:
            row = None
    else:
        row = results
    if isinstance(row, str):
        row = '"%s"' % row
    try:
        factory = WMFactory('webtools_factory')
        obj = factory.loadObject(self.config.model.object, self.config)
        res_version = obj.version
    except Exception:
        res_version = 'unknown'

    keyhash = hashlib.md5()

    if not isinstance(results, (str, bytes)):
        keyhash.update(encodeUnicodeToBytesConditional(str(results), condition=PY3))
    else:
        keyhash.update(encodeUnicodeToBytesConditional(results, condition=PY3))
    res_checksum = keyhash.hexdigest()
    dasdata = {'application': '%s.%s' % (self.config.application, func.__name__),
               'request_timestamp': start_time,
               'request_url': request.base + request.path_info + '?' + request.query_string,
               'request_method': request.method,
               'request_params': request.params,
               'response_version': res_version,
               'response_expires': res_expire,
               'response_checksum': res_checksum,
               'request_call': func.__name__,
               'call_time': call_time,
               'results': results,
               }
    return dasdata


def make_timestamp(seconds=0):
    """
    Convenience function to make a timestamp
    """
    then = datetime.now() + timedelta(seconds=seconds)
    return mktime(then.timetuple())


def make_rfc_timestamp(seconds=0):
    """
    Convenience function to make an rfc formatted timestamp
    """
    return format_date_time(make_timestamp(seconds))



# -*- coding: utf-8 -*-
"""
_SetPendingSlots_

Oracle implementation of Locations.SetPendingSlots
Created on Mon Jun 18 13:59:24 2012

@author: dballest
"""

from WMCore.WMBS.MySQL.Locations.SetPendingSlots import SetPendingSlots as MySQLSetPendingSlots

class SetPendingSlots(MySQLSetPendingSlots):
    """
    Same as MySQL version
    """
    pass



#!/usr/bin/env python
"""
_Status_

MySQL implementation of DBSBuffer.Status
"""




from WMCore.Database.DBFormatter import DBFormatter

class Status(DBFormatter):
    """
    _Status_

    Retrieve information about the size and number of events contained in
    datasets in DBSBuffer.
    """
    sql = """SELECT dbsbuffer_dataset.path, dbsbuffer_algo.app_name, dbsbuffer_algo.app_ver,
                    SUM(events) AS events, SUM(filesize) AS filesize FROM dbsbuffer_file
               INNER JOIN dbsbuffer_algo_dataset_assoc ON
                 dbsbuffer_file.dataset_algo = dbsbuffer_algo_dataset_assoc.id
               INNER JOIN dbsbuffer_dataset ON
                 dbsbuffer_algo_dataset_assoc.dataset_id = dbsbuffer_dataset.id
               INNER JOIN dbsbuffer_algo ON
                 dbsbuffer_algo_dataset_assoc.algo_id = dbsbuffer_algo.id
               WHERE dbsbuffer_dataset.path != "bogus"
               GROUP BY dbsbuffer_dataset.path, dbsbuffer_algo.app_name, dbsbuffer_algo.app_ver"""

    def converDecimalToInt(self, results):
        for result in results:
            if result['events'] != None:
                result['events'] = int(result['events'])
            if result['filesize'] != None:
                result['filesize'] = int(result['filesize'])
        return results

    def execute(self, conn = None, transaction = False):
        results = self.dbi.processData(self.sql, conn = conn, transaction = transaction)
        results = self.formatDict(results)
        self.converDecimalToInt(results)
        return results



from builtins import str as newstr
import random, cherrypy

class RESTError(Exception):
    """Base class for REST errors.

    .. attribute:: http_code

       Integer, HTTP status code for this error. Also emitted as X-Error-HTTP
       header value.

    .. attribute:: app_code

       Integer, application error code, to be emitted as X-REST-Status header.

    .. attribute:: message

       String, information about the error, to be emitted as X-Error-Detail
       header. Should not contain anything sensitive, and in particular should
       never include any unvalidated or unsafe data, e.g. input parameters or
       data from a database. Normally a fixed label with one-to-one match with
       the :obj:`app-code`. If the text exceeds 200 characters, it's truncated.
       Since this is emitted as a HTTP header, it cannot contain newlines or
       anything encoding-dependent.

    .. attribute:: info

       String, additional information beyond :obj:`message`, to be emitted as
       X-Error-Info header. Like :obj:`message` should not contain anything
       sensitive or unsafe, or text inappropriate for a HTTP response header,
       and should be short enough to fit in 200 characters. This is normally
       free form text to clarify why the error happened.

    .. attribute:: errid

       String, random unique identifier for this error, to be emitted as
       X-Error-ID header and output into server logs when logging the error.
       The purpose is that clients save this id when they receive an error,
       and further error reporting or debugging can use this value to identify
       the specific error, and for example to grep logs for more information.

    .. attribute:: errobj

       If the problem was caused by another exception being raised in the code,
       reference to the original exception object. For example if the code dies
       with an :class:`KeyError`, this is the original exception object. This
       error is logged to the server logs when reporting the error, but no
       information about it is returned to the HTTP client.

    .. attribute:: trace

       The origin of the exception as returned by :func:`format_exc`. The full
       trace is emitted to the server logs, each line prefixed with timestamp.
       This information is not returned to the HTTP client.
    """

    http_code = None
    app_code = None
    message = None
    info = None
    errid = None
    errobj = None
    trace = None

    def __init__(self, info = None, errobj = None, trace = None):
        self.errid = "%032x" % random.randrange(1 << 128)
        self.errobj = errobj
        self.info = info
        self.trace = trace

    def __str__(self):
        return "%s %s [HTTP %d, APP %d, MSG %s, INFO %s, ERR %s]" \
          % (self.__class__.__name__, self.errid, self.http_code, self.app_code,
             repr(self.message).replace("\n", " ~~ "),
             repr(self.info).replace("\n", " ~~ "),
             repr(self.errobj).replace("\n", " ~~ "))

class NotAcceptable(RESTError):
    "Client did not specify format it accepts, or no compatible format was found."
    http_code = 406
    app_code = 201
    message = "Not acceptable"

class UnsupportedMethod(RESTError):
    "Client used HTTP request method which isn't supported for any API call."
    http_code = 405
    app_code = 202
    message = "Request method not supported"

class MethodWithoutQueryString(RESTError):
    "Client provided a query string which isn't acceptable for this request method."
    http_code = 405
    app_code = 203
    message = "Query arguments not supported for this request method"

class APIMethodMismatch(RESTError):
    """Both the API and HTTP request methods are supported, but not in that
    combination."""
    http_code = 405
    app_code = 204
    message = "API not supported for this request method"

class APINotSpecified(RESTError):
    "The request URL is missing API argument."
    http_code = 400
    app_code = 205
    message = "API not specified"

class NoSuchInstance(RESTError):
    """The request URL is missing instance argument or the specified instance
    does not exist."""
    http_code = 404
    app_code = 206
    message = "No such instance"

class APINotSupported(RESTError):
    "The request URL provides wrong API argument."
    http_code = 404
    app_code = 207
    message = "API not supported"

class DataCacheEmpty(RESTError):
    "The wmstats data cache has not be created."
    http_code = 503
    app_code = 208
    message = "DataCache is Empty"

class DatabaseError(RESTError):
    """Parent class for database-related errors.

    .. attribute: lastsql

       A tuple of *(sql, binds, kwbinds),* where `sql` is the last SQL statement
       executed and `binds`, `kwbinds` are the bind values used with it. Any
       sensitive parts like passwords have already been censored from the `sql`
       string. Note that for massive requests `binds` or `kwbinds` can get large.
       These are logged out in the server logs when reporting the error, but no
       information about these are returned to the HTTP client.

    .. attribute: intance

       String, the database instance for which the error occurred. This is
       reported in the error message output to server logs, but no information
       about this is returned to the HTTP client."""

    lastsql = None
    instance = None
    def __init__(self, info = None, errobj = None, trace = None,
                 lastsql = None, instance = None):
        RESTError.__init__(self, info, errobj, trace)
        self.lastsql = lastsql
        self.instance = instance

class DatabaseUnavailable(DatabaseError):
    """The instance argument is correct, but cannot connect to the database.
    This error will only occur at initial attempt to connect to the database,
    :class:`~.DatabaseConnectionError` is raised instead if the connection
    ends prematurely after the transaction has already begun successfully."""
    http_code = 503
    app_code = 401
    message = "Database unavailable"

class DatabaseConnectionError(DatabaseError):
    """Database was available when the operation started, but the connection
    was lost or otherwise failed during the application operation."""
    http_code = 504
    app_code = 402
    message = "Database connection failure"

class DatabaseExecutionError(DatabaseError):
    """Database operation failed."""
    http_code = 500
    app_code = 403
    message = "Execution error"

class MissingParameter(RESTError):
    "Client did not supply a parameter which is required."
    http_code = 400
    app_code = 301
    message = "Missing required parameter"

class InvalidParameter(RESTError):
    "Client supplied invalid value for a parameter."
    http_code = 400
    app_code = 302
    message = "Invalid input parameter"

class MissingObject(RESTError):
    """An object required for the operation is missing. This might be a
    pre-requisite needed to create a reference, or attempt to delete
    an object which does not exist."""
    http_code = 400
    app_code = 303
    message = "Required object is missing"

class TooManyObjects(RESTError):
    """Too many objects matched specified criteria. Usually this means
    more than one object was matched, deleted, or inserted, when only
    exactly one should have been subject to the operation."""
    http_code = 400
    app_code = 304
    message = "Too many objects"

class ObjectAlreadyExists(RESTError):
    """An already existing object is on the way of the operation. This
    is usually caused by uniqueness constraint violations when creating
    new objects."""
    http_code = 400
    app_code = 305
    message = "Object already exists"

class InvalidObject(RESTError):
    "The specified object is invalid."
    http_code = 400
    app_code = 306
    message = "Invalid object"

class ExecutionError(RESTError):
    """Input was in principle correct but there was an error processing
    the request. This normally means either programming error, timeout, or
    an unusual and unexpected problem with the database. For security reasons
    little additional information is returned. If the problem persists, client
    should contact service operators. The returned error id can be used as a
    reference."""
    http_code = 500
    app_code = 403
    message = "Execution error"

def report_error_header(header, val):
    """If `val` is non-empty, set CherryPy response `header` to `val`.
    Replaces all newlines with "; " characters. If the resulting value is
    longer than 200 characters, truncates it to the first 197 characters
    and leaves a trailing ellipsis "..."."""
    if val:
        val = val.replace("\n", "; ")
        if len(val) > 200: val = val[:197] + "..."
        cherrypy.response.headers[header] = val

def report_rest_error(err, trace, throw):
    """Report a REST error: generate an appropriate log message, set the
    response headers and raise an appropriate :class:`~.HTTPError`.

    Normally `throw` would be True to translate the exception `err` into
    a HTTP server error, but the function can also be called with `throw`
    set to False if the purpose is merely to log an exception message.

    :arg err: exception object.
    :arg trace: stack trace to use in case `err` doesn't have one.
    :arg throw: raise a :class:`~.HTTPError` if True."""
    if isinstance(err, DatabaseError) and err.errobj:
        offset = None
        sql, binds, kwbinds = err.lastsql
        if sql and err.errobj.args and hasattr(err.errobj.args[0], 'offset'):
            offset = err.errobj.args[0].offset
            sql = sql[:offset] + "<**>" + sql[offset:]
        cherrypy.log("SERVER DATABASE ERROR %d/%d %s %s.%s %s [instance: %s] (%s);"
                     " last statement: %s; binds: %s, %s; offset: %s"
                     % (err.http_code, err.app_code, err.message,
                        getattr(err.errobj, "__module__", "__builtins__"),
                        err.errobj.__class__.__name__,
                        err.errid, err.instance, newstr(err.errobj).rstrip(),
                        sql, binds, kwbinds, offset))
        for line in err.trace.rstrip().split("\n"): cherrypy.log("  " + line)
        cherrypy.response.headers["X-REST-Status"] = newstr(err.app_code)
        cherrypy.response.headers["X-Error-HTTP"] = newstr(err.http_code)
        cherrypy.response.headers["X-Error-ID"] = err.errid
        report_error_header("X-Error-Detail", err.message)
        report_error_header("X-Error-Info", err.info)
        if throw: raise cherrypy.HTTPError(err.http_code, err.message)
    elif isinstance(err, RESTError):
        if err.errobj:
            cherrypy.log("SERVER REST ERROR %s.%s %s (%s); derived from %s.%s (%s)"
                         % (err.__module__, err.__class__.__name__,
                            err.errid, err.message,
                            getattr(err.errobj, "__module__", "__builtins__"),
                            err.errobj.__class__.__name__,
                            newstr(err.errobj).rstrip()))
            trace = err.trace
        else:
            cherrypy.log("SERVER REST ERROR %s.%s %s (%s)"
                         % (err.__module__, err.__class__.__name__,
                            err.errid, err.message))
        for line in trace.rstrip().split("\n"): cherrypy.log("  " + line)
        cherrypy.response.headers["X-REST-Status"] = newstr(err.app_code)
        cherrypy.response.headers["X-Error-HTTP"] = newstr(err.http_code)
        cherrypy.response.headers["X-Error-ID"] = err.errid
        report_error_header("X-Error-Detail", err.message)
        report_error_header("X-Error-Info", err.info)
        if throw: raise cherrypy.HTTPError(err.http_code, err.message)
    elif isinstance(err, cherrypy.HTTPError):
        errid = "%032x" % random.randrange(1 << 128)
        cherrypy.log("SERVER HTTP ERROR %s.%s %s (%s)"
                     % (err.__module__, err.__class__.__name__,
                        errid, newstr(err).rstrip()))
        for line in trace.rstrip().split("\n"): cherrypy.log("  " + line)
        cherrypy.response.headers["X-REST-Status"] = newstr(200)
        cherrypy.response.headers["X-Error-HTTP"] = newstr(err.status)
        cherrypy.response.headers["X-Error-ID"] = errid
        report_error_header("X-Error-Detail", err._message)
        if throw: raise err
    else:
        errid = "%032x" % random.randrange(1 << 128)
        cherrypy.log("SERVER OTHER ERROR %s.%s %s (%s)"
                     % (getattr(err, "__module__", "__builtins__"),
                        err.__class__.__name__,
                        errid, newstr(err).rstrip()))
        for line in trace.rstrip().split("\n"): cherrypy.log("  " + line)
        cherrypy.response.headers["X-REST-Status"] = 400
        cherrypy.response.headers["X-Error-HTTP"] = 500
        cherrypy.response.headers["X-Error-ID"] = errid
        report_error_header("X-Error-Detail", "Server error")
        if throw: raise cherrypy.HTTPError(500, "Server error")



#!/usr/bin/env python
# encoding: utf-8
"""
DataCollectionService_t.py

Created by Dave Evans on 2010-10-05.
Copyright (c) 2010 Fermilab. All rights reserved.
"""

from builtins import range
import unittest

from Utils.PythonVersion import PY3

from nose.plugins.attrib import attr

from WMCore.ACDC.DataCollectionService import DataCollectionService, mergeFilesInfo
from WMCore.DataStructs.File import File
from WMCore.DataStructs.LumiList import LumiList
from WMCore.DataStructs.Run import Run
from WMCore.Services.UUIDLib import makeUUID
from WMCore.WMBS.Job import Job
from WMQuality.TestInitCouchApp import TestInitCouchApp


class DataCollectionService_t(unittest.TestCase):
    def setUp(self):
        """bootstrap tests"""
        self.testInit = TestInitCouchApp(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.testInit.setSchema(customModules=["WMCore.WMBS"],
                                useDefault=False)
        self.testInit.setupCouch("wmcore-acdc-datacollectionsvc", "GroupUser", "ACDC")

        if PY3:
            self.assertItemsEqual = self.assertCountEqual

        return

    def tearDown(self):
        self.testInit.tearDownCouch()
        self.testInit.clearDatabase()
        return

    @staticmethod
    def getMinimalJob(wf=None, task=None):
        job = Job()
        job["workflow"] = wf or "ACDCTest"
        job["task"] = task or "/ACDCTest/reco"
        job["location"] = "T1_US_FNAL_Disk"
        return job

    def testChunking(self):
        """
        _testChunking_

        Insert a workload and files that have several distinct sets of
        locations.  Verify that the chunks are created correctly and that they
        only groups files that have the same set of locations.  Also verify that
        the chunks are pulled out of ACDC correctly.
        """
        dcs = DataCollectionService(url=self.testInit.couchUrl, database="wmcore-acdc-datacollectionsvc")

        testFileA = File(lfn=makeUUID(), size=1024, events=1024)
        testFileA.setLocation(["cmssrm.fnal.gov", "castor.cern.ch"])
        testFileA.addRun(Run(1, 1, 2))
        testFileB = File(lfn=makeUUID(), size=1024, events=1024)
        testFileB.setLocation(["cmssrm.fnal.gov", "castor.cern.ch"])
        testFileB.addRun(Run(1, 3, 4))
        testFileC = File(lfn=makeUUID(), size=1024, events=1024)
        testFileC.setLocation(["cmssrm.fnal.gov", "castor.cern.ch"])
        testFileC.addRun(Run(1, 5, 6))
        testJobA = self.getMinimalJob()
        testJobA.addFile(testFileA)
        testJobA.addFile(testFileB)
        testJobA.addFile(testFileC)

        testFileD = File(lfn=makeUUID(), size=1024, events=1024)
        testFileD.setLocation(["cmssrm.fnal.gov"])
        testFileD.addRun(Run(2, 1, 2))
        testFileE = File(lfn=makeUUID(), size=1024, events=1024)
        testFileE.setLocation(["cmssrm.fnal.gov"])
        testFileE.addRun(Run(2, 3, 4))
        testJobB = self.getMinimalJob()
        testJobB.addFile(testFileD)
        testJobB.addFile(testFileE)

        testFileF = File(lfn=makeUUID(), size=1024, events=1024, parents={"/some/parent/F"})
        testFileF.setLocation(["cmssrm.fnal.gov", "castor.cern.ch", "srm.ral.uk"])
        testFileF.addRun(Run(3, 1, 2))
        testFileG = File(lfn=makeUUID(), size=1024, events=1024, parents={"/some/parent/G"})
        testFileG.setLocation(["cmssrm.fnal.gov", "castor.cern.ch", "srm.ral.uk"])
        testFileG.addRun(Run(3, 3, 4))
        testFileH = File(lfn=makeUUID(), size=1024, events=1024, parents={"/some/parent/H"})
        testFileH.setLocation(["cmssrm.fnal.gov", "castor.cern.ch", "srm.ral.uk"])
        testFileH.addRun(Run(3, 5, 6))
        testJobC = self.getMinimalJob()
        testJobC.addFile(testFileF)
        testJobC.addFile(testFileG)
        testJobC.addFile(testFileH)

        testFileI = File(lfn=makeUUID(), size=1024, events=1024, merged=True)
        testFileI.setLocation(["cmssrm.fnal.gov", "castor.cern.ch"])
        testFileI.addRun(Run(4, 1, 2))
        testFileJ = File(lfn=makeUUID(), size=1024, events=1024, merged=True)
        testFileJ.setLocation(["cmssrm.fnal.gov", "castor.cern.ch"])
        testFileJ.addRun(Run(4, 3, 4))
        testFileK = File(lfn=makeUUID(), size=1024, events=1024, merged=True)
        testFileK.setLocation(["cmssrm.fnal.gov", "castor.cern.ch"])
        testFileK.addRun(Run(4, 5, 6))
        testJobD = self.getMinimalJob()
        testJobD.addFile(testFileI)
        testJobD.addFile(testFileJ)
        testJobD.addFile(testFileK)

        dcs.failedJobs([testJobA, testJobB, testJobC, testJobD])
        chunks = dcs.chunkFileset("ACDCTest", "/ACDCTest/reco", chunkSize=5)

        self.assertEqual(len(chunks), 4, "Error: There should be four chunks: %s" % len(chunks))

        goldenMetaData = {1: {"lumis": 2, "locations": ["castor.cern.ch", "cmssrm.fnal.gov"], "events": 1024},
                          2: {"lumis": 4, "locations": ["cmssrm.fnal.gov"], "events": 2048},
                          3: {"lumis": 6, "locations": ["castor.cern.ch", "cmssrm.fnal.gov", "srm.ral.uk"],
                              "events": 3072},
                          5: {"lumis": 10, "locations": ["castor.cern.ch", "cmssrm.fnal.gov"], "events": 5120}}

        testFiles = [testFileA, testFileB, testFileC, testFileI, testFileJ, testFileK]
        lastFile = testFileA
        for testFile in testFiles:
            if lastFile["lfn"] < testFile["lfn"]:
                lastFile = testFile

        testFiles.remove(lastFile)

        goldenFiles = {1: [lastFile],
                       2: [testFileD, testFileE],
                       3: [testFileF, testFileG, testFileH],
                       5: testFiles}

        for chunk in chunks:
            chunkMetaData = dcs.getChunkInfo("ACDCTest", "/ACDCTest/reco",
                                             chunk["offset"], chunk["files"])

            self.assertEqual(chunkMetaData["files"], chunk["files"])
            self.assertEqual(chunkMetaData["lumis"], chunk["lumis"])
            self.assertEqual(chunkMetaData["events"], chunk["events"])
            self.assertEqual(chunkMetaData["locations"], chunk["locations"])

            self.assertTrue(chunk["files"] in goldenMetaData, "Error: Extra chunk found.")
            self.assertEqual(chunk["lumis"], goldenMetaData[chunk["files"]]["lumis"],
                             "Error: Lumis in chunk is wrong.")
            self.assertEqual(chunk["locations"], goldenMetaData[chunk["files"]]["locations"],
                             "Error: Locations in chunk is wrong.")
            self.assertEqual(chunk["events"], goldenMetaData[chunk["files"]]["events"],
                             "Error: Events in chunk is wrong.")
            del goldenMetaData[chunk["files"]]

            chunkFiles = dcs.getChunkFiles("ACDCTest", "/ACDCTest/reco", chunk["offset"], chunk["files"])

            self.assertTrue(chunk["files"] in goldenFiles, "Error: Extra chunk found.")
            goldenChunkFiles = goldenFiles[chunk["files"]]
            self.assertEqual(len(chunkFiles), len(goldenChunkFiles))

            for chunkFile in chunkFiles:
                foundFile = None
                for goldenChunkFile in goldenChunkFiles:
                    if chunkFile["lfn"] == goldenChunkFile["lfn"]:
                        foundFile = goldenChunkFile
                        break

                self.assertIsNotNone(foundFile, "Error: Missing chunk file: %s, %s" % (chunkFiles, goldenChunkFiles))
                self.assertEqual(set(foundFile["parents"]), chunkFile["parents"], "Error: File parents should match.")
                self.assertEqual(foundFile["merged"], chunkFile["merged"], "Error: File merged status should match.")
                self.assertEqual(foundFile["locations"], chunkFile["locations"], "Error: File locations should match.")
                self.assertEqual(foundFile["events"], chunkFile["events"])
                self.assertEqual(foundFile["size"], chunkFile["size"])
                self.assertEqual(len(foundFile["runs"]), len(chunkFile["runs"]), "Error: Wrong number of runs.")
                for run in foundFile["runs"]:
                    runMatch = False
                    for chunkRun in chunkFile["runs"]:
                        if chunkRun.run == run.run and chunkRun.lumis == run.lumis:
                            runMatch = True
                            break

                    self.assertTrue(runMatch, "Error: Run information is wrong.")

            del goldenFiles[chunk["files"]]

        singleChunk = dcs.singleChunkFileset("ACDCTest", "/ACDCTest/reco")
        self.assertEqual(singleChunk, {"offset": 0,
                                       "files": 11,
                                       "events": 11264,
                                       "lumis": 22,
                                       "locations": {"castor.cern.ch", "cmssrm.fnal.gov", "srm.ral.uk"}},
                         "Error: Single chunk metadata is wrong")

        return

    def testGetLumiWhitelist(self):
        """
        _testGetLumiWhitelist_

        Verify that the ACDC whitelist generation code works correctly.  We'll
        add jobs with the following lumi info:
          # Run 1, lumis [1, 2, 3], [4, 6], [7], [9], [11, 12]
          # Run 2, lumis [5, 6, 7], [10, 11, 12], [15]
          # Run 3, lumis [20]

        And should get out a whitelist that looks like this:
          {"1": [[1, 4], [6, 7], [9, 9], [11, 12]],
           "2": [[5, 7], [10, 12], [15, 15]],
           "3": [[20, 20]]}
        """
        dcs = DataCollectionService(url=self.testInit.couchUrl, database="wmcore-acdc-datacollectionsvc")

        testFileA = File(lfn=makeUUID(), size=1024, events=1024)
        testFileA.addRun(Run(1, 1, 2))
        testFileB = File(lfn=makeUUID(), size=1024, events=1024)
        testFileB.addRun(Run(1, 3))
        testJobA = self.getMinimalJob()
        testJobA.addFile(testFileA)
        testJobA.addFile(testFileB)

        testFileC = File(lfn=makeUUID(), size=1024, events=1024)
        testFileC.addRun(Run(1, 4, 6))
        testJobB = self.getMinimalJob()
        testJobB.addFile(testFileC)

        testFileD = File(lfn=makeUUID(), size=1024, events=1024)
        testFileD.addRun(Run(1, 7))
        testJobC = self.getMinimalJob()
        testJobC.addFile(testFileD)

        testFileE = File(lfn=makeUUID(), size=1024, events=1024)
        testFileE.addRun(Run(1, 11, 12))
        testJobD = self.getMinimalJob()
        testJobD.addFile(testFileE)

        testFileF = File(lfn=makeUUID(), size=1024, events=1024)
        testFileF.addRun(Run(2, 5, 6, 7))
        testJobE = self.getMinimalJob()
        testJobE.addFile(testFileF)

        testFileG = File(lfn=makeUUID(), size=1024, events=1024)
        testFileG.addRun(Run(2, 10, 11, 12))
        testJobF = self.getMinimalJob()
        testJobF.addFile(testFileG)

        testFileH = File(lfn=makeUUID(), size=1024, events=1024)
        testFileH.addRun(Run(2, 15))
        testJobG = self.getMinimalJob()
        testJobG.addFile(testFileH)

        testFileI = File(lfn=makeUUID(), size=1024, events=1024)
        testFileI.addRun(Run(3, 20))
        testJobH = self.getMinimalJob()
        testJobH.addFile(testFileI)

        testFileJ = File(lfn=makeUUID(), size=1024, events=1024)
        testFileJ.addRun(Run(1, 9))
        testJobI = self.getMinimalJob()
        testJobI.addFile(testFileJ)

        dcs.failedJobs([testJobA, testJobB, testJobC, testJobD, testJobE,
                        testJobF, testJobG, testJobH, testJobI])
        whiteList = dcs.getLumiWhitelist("ACDCTest", "/ACDCTest/reco")

        self.assertEqual(len(whiteList), 3,
                         "Error: There should be 3 runs.")
        self.assertEqual(whiteList["1"], [[1, 4], [6, 7], [9, 9], [11, 12]],
                         "Error: Whitelist for run 1 is wrong.")
        self.assertEqual(whiteList["2"], [[5, 7], [10, 12], [15, 15]],
                         "Error: Whitelist for run 2 is wrong.")
        self.assertEqual(whiteList["3"], [[20, 20]],
                         "Error: Whitelist for run 3 is wrong.")

        correctLumiList = LumiList(compactList={"1": [[1, 4], [6, 7], [9, 9], [11, 12]],
                                                "2": [[5, 7], [10, 12], [15, 15]],
                                                "3": [[20, 20]]})
        testLumiList = dcs.getLumilistWhitelist("ACDCTest", "/ACDCTest/reco")
        self.assertEqual(correctLumiList.getCMSSWString(), testLumiList.getCMSSWString())

        return

    def testFakeMergeFilesInfo(self):
        """
        _testFakeMergeFilesInfo_

        Verify that we can merge MCFakeFiles together when a fileset contains
        several failures for the same input fake file.
        """
        fakeFiles = [{'checksums': {},
                      'events': 500000,
                      'first_event': 1,
                      'id': 40,
                      'last_event': 0,
                      'lfn': 'MCFakeFile-File1',
                      'locations': ['T1_DE_KIT_Disk'],
                      'merged': '0',
                      'parents': [],
                      'runs': [{'lumis': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11], 'run_number': 1}],
                      'size': 0},
                     {'checksums': {},
                      'events': 500000,
                      'first_event': 1000001,
                      'id': 40,
                      'last_event': 0,
                      'lfn': 'MCFakeFile-File2',
                      'locations': ['T1_DE_KIT_Disk'],
                      'merged': '0',
                      'parents': [],
                      'runs': [{'lumis': [21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31],
                                'run_number': 1}],
                      'size': 0},
                     {'checksums': {},
                      'events': 500000,
                      'first_event': 2000001,
                      'id': 40,
                      'last_event': 0,
                      'lfn': 'MCFakeFile-File1',
                      'locations': ['T1_DE_KIT_Disk'],
                      'merged': '0',
                      'parents': [],
                      'runs': [{'lumis': [41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51],
                                'run_number': 1}],
                      'size': 0},
                     {'checksums': {},
                      'events': 500000,
                      'first_event': 7000001,
                      'id': 40,
                      'last_event': 0,
                      'lfn': 'MCFakeFile-File3',
                      'locations': ['T1_DE_KIT_Disk'],
                      'merged': '0',
                      'parents': [],
                      'runs': [{'lumis': [81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91],
                                'run_number': 1}],
                      'size': 0},
                     {'checksums': {},
                      'events': 500000,
                      'first_event': 4000001,
                      'id': 40,
                      'last_event': 0,
                      'lfn': 'MCFakeFile-File3',
                      'locations': ['T1_DE_KIT_Disk'],
                      'merged': '0',
                      'parents': [],
                      'runs': [{'lumis': [51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61],
                                'run_number': 1}],
                      'size': 0}]

        mergedFiles = mergeFilesInfo(fakeFiles)
        self.assertEqual(len(mergedFiles), 3, "Error: wrong number of files.")

        totalEvents = 0
        for job in mergedFiles:
            totalEvents += job['events']
        self.assertEqual(totalEvents, 2500000, "Error: wrong number of total events.")

        for job in mergedFiles:
            if job['lfn'] == 'MCFakeFile-File1':
                lumiList = job['runs'][0]['lumis']
                self.assertEqual(len(lumiList), 22)

    def testDataMergeFilesInfo(self):
        """
        _testDataMergeFilesInfo_

        Verify that we can properly merge real input files coming from
        the ACDCServer.
        """
        originalFiles = [{u'events': 165,
                          u'lfn': u'/store/unmerged/fileA.root',
                          u'locations': [u'T2_CH_CERN', u'T2_CH_CERNBOX'],
                          u'parents': [],
                          u'runs': [{u'lumis': [1810823], u'run_number': 1}]},
                         {u'events': 165,
                          u'lfn': u'/store/unmerged/fileA.root',
                          u'locations': [u'T2_CH_CERN', u'T2_CH_CERNBOX'],
                          u'parents': [],
                          u'runs': [{u'lumis': [1810823], u'run_number': 2}]},
                         {u'events': 50,
                          u'lfn': u'/store/unmerged/fileB.root',
                          u'locations': [u'T1_US_FNAL_MSS'],
                          u'parents': ['file1', 'file3'],
                          u'runs': [{u'lumis': [1, 2, 3], u'run_number': 1}]},
                         {u'events': 165,
                          u'lfn': u'/store/unmerged/fileA.root',
                          u'locations': [u'T2_CH_CERN', u'T2_CH_CERNBOX'],
                          u'parents': [],
                          u'runs': [{u'lumis': [1810824], u'run_number': 1}]},
                         {u'events': 50,
                          u'lfn': u'/store/unmerged/fileB.root',
                          u'locations': [u'T1_US_FNAL_MSS'],
                          u'parents': ['file2'],
                          u'runs': [{u'lumis': [4, 5, 6], u'run_number': 1},
                                    {u'lumis': [9], u'run_number': 1},
                                    {u'lumis': [7, 8], u'run_number': 7}]},
                         {u'events': 10,
                          u'lfn': u'/store/unmerged/fileC.root',
                          u'locations': [u'T1_US_FNAL_Disk', u'T2_US_MIT'],
                          u'parents': [],
                          u'runs': [{u'lumis': [111], u'run_number': 222}]}]

        mergedFiles = mergeFilesInfo(originalFiles)
        self.assertEqual(len(mergedFiles), 3, "Error: wrong number of files.")
        self.assertItemsEqual([i['lfn'] for i in mergedFiles], ['/store/unmerged/fileA.root',
                                                                '/store/unmerged/fileB.root',
                                                                '/store/unmerged/fileC.root'])

        for item in mergedFiles:
            if item['lfn'] == '/store/unmerged/fileA.root':
                self.assertEqual(item['events'], 165)
                self.assertItemsEqual(item['locations'], ['T2_CH_CERN', 'T2_CH_CERNBOX'])
                for runLumi in item['runs']:
                    if runLumi['run_number'] == 1:
                        self.assertItemsEqual(runLumi['lumis'], [1810824, 1810823])
                    elif runLumi['run_number'] == 2:
                        self.assertItemsEqual(runLumi['lumis'], [1810823])
                    else:
                        raise AssertionError("This should never happen")
                self.assertEqual(item['parents'], [])
            elif item['lfn'] == '/store/unmerged/fileB.root':
                self.assertEqual(item['events'], 50)
                self.assertItemsEqual(item['locations'], ['T1_US_FNAL_MSS'])
                for runLumi in item['runs']:
                    if runLumi['run_number'] == 1:
                        self.assertItemsEqual(runLumi['lumis'], [1, 2, 3, 4, 5, 6, 9])
                    elif runLumi['run_number'] == 7:
                        self.assertItemsEqual(runLumi['lumis'], [7, 8])
                    else:
                        raise AssertionError("This should never happen")
                self.assertItemsEqual(item['parents'], ['file1', 'file2', 'file3'])
            elif item['lfn'] == '/store/unmerged/fileC.root':
                self.assertEqual(item['events'], 10)
                self.assertItemsEqual(item['locations'], ['T1_US_FNAL_Disk', 'T2_US_MIT'])
                self.assertItemsEqual(item['runs'], [{'lumis': [111], 'run_number': 222}])
                self.assertEqual(item['parents'], [])

    def jobConfig(self, wf, task, jobid, lfn):
        """
        Create a fake job dict to upload to the ACDC server
        """
        testFile = File(lfn=lfn, size=1024, events=1024)
        testFile.setLocation(["T2_CH_CERN", "T2_CH_CERN_HLT"])
        testFile.addRun(Run(jobid, 1, 2))  # run = jobid
        testJob = self.getMinimalJob(wf, task)
        testJob.addFile(testFile)

        return testJob

    @attr('integration')
    def testFailedJobsUniqueWf(self):
        """
        Performance test of failedJobs with all failed jobs belonging
        to the same workflow and the same task name
        """
        loadList = []
        for i in range(1, 5000):
            loadList.append(self.jobConfig('wf1', '/wf1/task1', i, 'lfn1'))

        dcs = DataCollectionService(url=self.testInit.couchUrl, database="wmcore-acdc-datacollectionsvc")
        dcs.failedJobs(loadList)
        return

    @attr('integration')
    def testFailedJobsScrambledWf(self):
        """
        Performance test of failedJobs where jobs belong to 10 different
        workflows and 3 different tasks
        """
        loadList = []
        for i in range(1, 5000):
            wfName = "wf%d" % (i % 10)
            taskName = "/wf%d/task%d" % (i % 10, i % 3)
            loadList.append(self.jobConfig(wfName, taskName, i, '/file/name/lfn1'))

        dcs = DataCollectionService(url=self.testInit.couchUrl, database="wmcore-acdc-datacollectionsvc")
        dcs.failedJobs(loadList)
        return


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
_MyProxy_t_
Test the basic MyProxy operations.
You need to source your UI before running these tests.
The user Proxy and MyProxy is initialized in testCreateMyProxy method and they are used by the remaining tests.
"""
from __future__ import division

import unittest
import os
import logging
import logging.config
import time

from nose.plugins.attrib import attr
from WMCore.Credential.Proxy import Proxy, myProxyEnvironment

# You may have to change these variables to run in a local environment
group = os.environ.get('PROXY_GROUP', '')
role = os.environ.get('PROXY_ROLE', 'NULL')
myProxySvr = os.environ.get('MYPROXY_SERVER', 'myproxy.cern.ch')
uiPath = os.environ.get('GLITE_UI', '/afs/cern.ch/cms/LCG/LCG-2/UI/cms_ui_env.sh')

serverDN = os.environ.get('SERVER_DN', '/C=IT/O=INFN/OU=Host/L=Perugia/CN=crab.pg.infn.it')
serverKey = os.environ.get('SERVER_KEY', '/path/to/key.pem')
serverCert = os.environ.get('SERVER_CERT', '/path/to/cert.pem')

class MyProxyTest(unittest.TestCase):

    def setUp(self):
        """
        Setup for unit tests
        """
        logging.basicConfig(level=logging.DEBUG,
                            format='%(asctime)s %(name)-12s %(levelname)-8s %(message)s',
                            datefmt='%m-%d %H:%M',
                            filename='proxy_unittests.log',
                            filemode='w')
        logger_name = 'ProxyTest'
        self.logger = logging.getLogger(logger_name)
        self.dict = {'logger': self.logger, 'vo': 'cms', 'group': group, 'role': role,
                     'myProxySvr': myProxySvr, 'proxyValidity' : '192:00', 'min_time_left' : 36000,
                     'uisource' : uiPath, 'serverDN' : serverDN}

        self.proxyPath = None
        self.proxy = Proxy( self.dict )
        self.serverDN = self.dict['serverDN']

    def tearDown(self):
        """
        _tearDown_
        """
        return

    @attr("integration")
    def testAAACreateMyProxy( self ):
        """
        Test if delegate method create correctly the MyProxy.
        """
        self.proxy.create()
        self.proxy.delegate( credential = self.proxyPath )
        valid = self.proxy.checkMyProxy( )
        self.assertTrue(valid, 'Could not create MyProxy')

    @attr("integration")
    def testDelegateServer( self ):
        """
        Test if delegate method create MyProxy and delegate
        the retrieval to the server correctly.
        """
        self.proxy.delegate( credential = self.proxyPath, serverRenewer = True )
        valid = self.proxy.checkMyProxy( checkRenewer = True )
        self.assertTrue(valid)

    @attr("integration")
    def testCheckMyProxy( self ):
        """
        Test if checkMyProxy checks correctly the MyProxy validity.
        """
        valid = self.proxy.checkMyProxy( )
        self.assertTrue(valid)

    @attr("integration")
    def testRenewMyProxy( self ):
        """
        Test if renewMyProxy method renews correctly the MyProxy.
        """
        self.proxy.renewMyProxy( proxy = self.proxyPath )
        time.sleep( 5 )
        timeLeft = self.proxy.getMyProxyTimeLeft( proxy = self.proxyPath )
        self.assertEqual(int(int(timeLeft) // 3600), 167)

    @attr("integration")
    def testRenewMyProxyForServer( self ):
        """
        Renew MyProxy which the retrieval is delegated to a server.
        """
        time.sleep( 70 )
        self.proxy.renewMyProxy( proxy = self.proxyPath, serverRenewer = True )
        time.sleep( 5 )
        timeLeft = self.proxy.getMyProxyTimeLeft( proxy = self.proxyPath, serverRenewer = True )
        self.assertEqual(int(int(timeLeft) // 3600), 167)

    @attr("integration")
    def testMyProxyEnvironment(self):
        """
        Test the myProxyEnvironment context manager
        In this test a new Proxy and MyProxy are initialized
        """
        myProxy = Proxy(self.dict)

        # Create the proxy
        myProxy.create()
        proxyPath = myProxy.getProxyFilename()
        userDN = myProxy.getSubject()
        self.assertTrue(os.path.exists(proxyPath))

        # Delegate and check the proxy
        myProxy.delegate(credential=proxyPath, serverRenewer=True)
        valid = myProxy.checkMyProxy()
        self.assertTrue(valid)

        # Make sure X509_USER_PROXY exists only in the context manager and corresponds to a file
        if 'X509_USER_PROXY' in os.environ:
            del os.environ['X509_USER_PROXY']
        self.assertFalse('X509_USER_PROXY' in os.environ)
        with myProxyEnvironment(userDN=userDN, serverCert=serverCert, serverKey=serverKey,
                                myproxySrv='myproxy.cern.ch', proxyDir='/tmp/', logger=self.logger):
            self.assertTrue('X509_USER_PROXY' in os.environ)
            self.assertTrue(os.path.exists(os.environ['X509_USER_PROXY']))
        self.assertFalse('X509_USER_PROXY' in os.environ)

        return

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
_TwoFileAndEventBased_

"""





from WMCore.JobSplitting.JobFactory import JobFactory
from WMCore.Services.UUIDLib import makeUUID

class TwoFileAndEventBased(JobFactory):
    """
    Split jobs by number of events
    """
    def algorithm(self, *args, **kwargs):
        """
        _algorithm_

        An event base splitting algorithm.  All available files are split into a
        set number of events per job.
        """
        jobGroups = []
        fileset = self.subscription.availableFiles()

        #  //
        # // get the event total
        #//
        eventsPerJob = int(kwargs.get("events_per_job", 5000))

        try:
            selectionAlgorithm = kwargs['selection_algorithm']
        except KeyError as e:
            selectionAlgorithm = None
        carryOver = 0

        for f in fileset:
            if selectionAlgorithm:
                if not selectionAlgorithm( f ):
                    self.subscription.completeFiles( [ f ] )
                    continue
            f.loadData(parentage = 1)
            self.newGroup()
            eventsInFile = f['events']

            if eventsInFile == 0:
                self.newJob(name = makeUUID())
                self.currentJob.addFile(f)
                continue

            currentEvent = 0
            while currentEvent < eventsInFile:
                self.newJob(name = makeUUID())
                self.currentJob.addFile(f)
                self.currentJob["mask"].setMaxAndSkipEvents(eventsPerJob, currentEvent)
                currentEvent += eventsPerJob



from __future__ import (print_function, division)

import os
import unittest

from mock import  mock

from Utils.PythonVersion import PY3

from WMCore.Storage.Execute import execute, runCommand, runCommandWithOutput
from WMCore.Storage.StageOutError import StageOutError
from WMCore.WMBase import getTestBase


class ExecuteTest(unittest.TestCase):
    base = os.path.join(getTestBase(), "WMCore_t/Storage_t/ExecutableCommands.py")

    def setUp(self):
        self.python_runtime = "python3" if PY3 else "python"

    @mock.patch('WMCore.Storage.Execute.runCommandWithOutput')
    def testExecute_exception(self, execute_runCommand):
        execute_runCommand.return_value = Exception("Im in test, yay!")
        self.assertRaises(StageOutError, execute, "test")

    @mock.patch('WMCore.Storage.Execute.runCommandWithOutput')
    def testExecute_exitCode(self, execute_runCommand):
        execute_runCommand.return_value = "Im exitCode", "test output"
        self.assertRaises(StageOutError, execute, "test")

    @mock.patch('WMCore.Storage.Execute.runCommandWithOutput')
    def testExecute_executed(self, execute_runCommand):
        execute_runCommand.return_value = 0, "test output"
        self.assertIsNone(execute("test"))
        execute_runCommand.assert_called_with("test")

    def testRunCommand_results(self):
        self.assertEqual(1, runCommand("%s %s -exit 0" % (self.python_runtime, self.base)))
        self.assertEqual(1, runCommand("%s %s -exit a" % (self.python_runtime, self.base)))
        self.assertEqual(2, runCommand("%s %s -text" % (self.python_runtime, self.base)))
        self.assertEqual(0, runCommand("%s %s -text a" % (self.python_runtime, self.base)))

    def testRunCommandWithOutput_results(self):
        err, text = runCommandWithOutput("%s %s -text" % (self.python_runtime, self.base))
        self.assertEqual(2, err)
        self.assertTrue("ExecutableCommands.py: error" in text)
        self.assertEqual((1, 'stdout: \nstderr: 0\n'), runCommandWithOutput("%s %s -exit 0" % (self.python_runtime, self.base)))
        self.assertEqual((1, 'stdout: \nstderr: a\n'), runCommandWithOutput("%s %s -exit a" % (self.python_runtime, self.base)))
        self.assertEqual((0, 'stdout: a\n\nstderr: '), runCommandWithOutput("%s %s -text a" % (self.python_runtime, self.base)))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Script to be executed with the WMAgent environment, does you first need to execute:
source apps/wmagent/etc/profile.d/init.sh

Provided a workflow name in the command line, it will find all the
local workqueue elements and print a summary of work/data location
for the elements sitting in the Available status
"""
from __future__ import print_function, division

from builtins import next
from future.utils import viewvalues, listvalues

import sys
import os

from WMCore.Configuration import loadConfigurationFile
from WMCore.WorkQueue.WorkQueueBackend import WorkQueueBackend
from WMCore.WorkQueue.DataStructs.WorkQueueElement import possibleSites


def printElementsSummary(reqName, elements, queueUrl):
    """
    Print the local couchdb situation based on the WQE status
    """
    print("Summary for %s and request %s" % (queueUrl, reqName))
    for elem in elements:
        if elem['Status'] != "Available":
            continue
        targetSites = possibleSites(elem)
        commonDataLoc = commonDataLocation(elem)
        print("  Element '%s' has the following site intersection: %s, with common data location: %s"
              % (elem.id, targetSites, commonDataLoc))
        printDataLocation(elem)

        if not targetSites and commonDataLoc:
            print("    this workflow has to be assigned to: %s" % commonDataLoc)
        if not targetSites and not commonDataLoc:
            print("    this workflow has to be assigned with AAA flags enabled according to input/PU location")


def printDataLocation(element):
    """
    Print all the input data and their current location
    """
    print("    LQE has primary AAA: %s and secondary AAA: %s" % (element['NoInputUpdate'], element['NoPileupUpdate']))
    print("    LQE has Inputs: %s" % element['Inputs'])
    print("    LQE has ParentData: %s" % element['ParentData'])
    print("    LQE has PileupData: %s" % element['PileupData'])

def commonDataLocation(element):
    """
    Make an intersection of all the data location
    :param element: workqueue element object
    :return: list with the common **data** location
    """
    commonLoc = set()
    if element['PileupData']:
        commonLoc = set(next(iter(viewvalues(element['PileupData']))))
    if element['Inputs']:
        if commonLoc:
            commonLoc = commonLoc & set(next(iter(viewvalues(element['Inputs']))))
        else:
            commonLoc = set(list(element['Inputs'].values())[0])
    if element['ParentData']:
        tempLoc = listvalues(element['ParentData'])
        parentLoc = set(tempLoc[0])
        for temp in tempLoc:
            parentLoc = parentLoc & set(temp)
        commonLoc = commonLoc & parentLoc
    return commonLoc

def main():
    if 'WMAGENT_CONFIG' not in os.environ:
        os.environ['WMAGENT_CONFIG'] = '/data/srv/wmagent/current/config/wmagent/config.py'
    config = loadConfigurationFile(os.environ["WMAGENT_CONFIG"])

    if len(sys.argv) != 2:
        print("You must provide a request name")
        sys.exit(1)
    reqName = sys.argv[1]

    localWQBackend = WorkQueueBackend(config.WorkQueueManager.couchurl, db_name="workqueue")
    localDocIDs = localWQBackend.getElements(WorkflowName=reqName)
    printElementsSummary(reqName, localDocIDs, localWQBackend.queueUrl)

    sys.exit(0)

if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python
# encoding: utf-8
"""
cmsRun.py

Created by Dave Evans on 2010-03-16.
Copyright (c) 2010 Fermilab. All rights reserved.

cmsRun simulator for unittest purposes.

If you are a physicist, use this instead of the real cmsRun and
complain that it doesnt make the events you ask for, I will kneecap you

"""

import sys

class CmsRun:
    """
    _CmsRun_

    Emulate cmsRun behaviour

    """
    def __init__(self, *args):
        self.args = list(args)
        open("BogusFile.txt", "w").close()

if __name__ == '__main__':
    cmsrun = CmsRun(*sys.argv[1:])



'''
Created on Jun 18, 2009

@author: meloam
'''
import unittest
import os

import WMCore.Storage.StageOutMgr as StageOutMgr

class StageOutMgrTest(unittest.TestCase):

    def setUp(self):
        # shut up SiteLocalConfig
        os.putenv('CMS_PATH', os.getcwd())

    def testName(self):
        pass


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



#!/usr/bin/env python
"""
_LoadByWMBSID_

Oracle implementation for loading a job by WMBS info
"""


from WMCore.BossAir.MySQL.LoadByWMBSID import LoadByWMBSID as MySQLLoadByWMBSID

class LoadByWMBSID(MySQLLoadByWMBSID):
    """
    _LoadByWMBSID_

    Load all jobs in full by WMBS ID and retry count
    """



#!/usr/bin/env python
"""
    CouchWorkQueueElement unit tests
"""

import unittest
from WMQuality.TestInitCouchApp import TestInitCouchApp as TestInit

from WMCore.WorkQueue.DataStructs.CouchWorkQueueElement import CouchWorkQueueElement, fixElementConflicts



class CouchWorkQueueElementTest(unittest.TestCase):

    def setUp(self):
        self.testInit = TestInit('CouchWorkQueueTest')
        self.testInit.setLogging()
        self.testInit.setupCouch('couch_wq_test')
        self.couch_db = self.testInit.couch.couchServer.connectDatabase('couch_wq_test')

    def tearDown(self):
        """
        _tearDown_

        """
        self.testInit.tearDownCouch()

    def testIdSaved(self):
        """Generated id used as db id"""
        ele = CouchWorkQueueElement(self.couch_db, elementParams = {'RequestName' : 'test'})
        ele.save()
        self.couch_db.commit(timestamp = True)
        self.assertTrue(self.couch_db.documentExists(ele.id))
        self.assertEqual(self.couch_db.info()['doc_count'], 1)

    def testIdFromDbImmutable(self):
        """Modifying element id algorithm doesn't change existing id's"""
        ele = CouchWorkQueueElement(self.couch_db, elementParams = {'RequestName' : 'test'})
        ele.save()
        self.couch_db.commit(timestamp = True)
        ele2 = CouchWorkQueueElement(self.couch_db, id = ele.id).load()
        ele2['RequestName'] = 'ThisWouldCauseIdToChange'
        # id should not change
        self.assertEqual(ele.id, ele2.id)
        # save should modify existing element
        ele2.save()
        self.couch_db.commit(timestamp = True)
        self.assertEqual(self.couch_db.info()['doc_count'], 1)

    def testFixElementConflicts(self):
        """Element conflicts handled"""
        # list of before and after elements for comparison
        values = [
                  # status conflict
                  [[{'RequestName' : 'arequest', 'Status' : 'Available'}, {'RequestName' : 'arequest', 'Status' : 'CancelRequested'}],
                  [{'RequestName' : 'arequest', 'Status' : 'CancelRequested'}, {'RequestName' : 'arequest', 'Status' : 'CancelRequested'}]
                 ],
                  # location conflict - local uses location and global subscriptions
                  [[{'RequestName' : 'brequest', 'Status' : 'CancelRequested', 'Inputs' : {'a' : [1]}}, {'RequestName' : 'brequest', 'Status' : 'Available', 'Inputs' : {'a' : [1,2]}}],
                  [{'RequestName' : 'brequest', 'Status' : 'CancelRequested', 'Inputs' : {'a' : [1,2]}}, {'RequestName' : 'brequest', 'Status' : 'Available', 'Inputs' : {'a' : [1,2]}}]
                 ],
                  # status and progress conflict
                  [[{'RequestName' : 'crequest', 'Status' : 'Available', 'PercentComplete' : 69}, {'RequestName' : 'crequest', 'Status' : 'CancelRequested'}],
                  [{'RequestName' : 'crequest', 'Status' : 'CancelRequested', 'PercentComplete' : 69}, {'RequestName' : 'crequest', 'Status' : 'CancelRequested'}]
                 ],
                  # status and subscription conflict
                  [[{'RequestName' : 'drequest', 'Status' : 'Running', 'SubscriptionId' : 69}, {'RequestName' : 'drequest', 'Status' : 'CancelRequested'}],
                  [{'RequestName' : 'drequest', 'Status' : 'CancelRequested', 'SubscriptionId' : 69}, {'RequestName' : 'drequest', 'Status' : 'CancelRequested'}]
                 ],
                  # whitelist conflict
                  [[{'RequestName' : 'erequest', 'SiteWhitelist' : [1,2]}, {'RequestName' : 'erequest'}],
                  [{'RequestName' : 'erequest', 'SiteWhitelist' : [1,2]}, {'RequestName' : 'erequest'}]
                 ],
                  # black conflict
                  [[{'RequestName' : 'frequest'}, {'RequestName' : 'frequest', 'SiteBlacklist' : [1,2]}],
                  [{'RequestName' : 'frequest', 'SiteBlacklist' : [1,2]}, {'RequestName' : 'frequest', 'SiteBlacklist' : [1,2]}]
                 ],
                  # priority conflict
                  [[{'RequestName' : 'grequest'}, {'RequestName' : 'grequest', 'Priority' : 69}],
                  [{'RequestName' : 'grequest', 'Priority' : 69}, {'RequestName' : 'grequest', 'Priority' : 69}]
                 ],
                ]
        for before, after in values:
            before = [CouchWorkQueueElement(self.couch_db, 1, elementParams = x) for x in before]
            # add fake revs
            [x._document.__setitem__('_rev', "a") for x in before]
            after = [CouchWorkQueueElement(self.couch_db, 1, elementParams = x) for x in after]
            self.assertEqual(list(fixElementConflicts(*before)), after)

if __name__ == '__main__':
    unittest.main()



"""
Created on Jun 18, 2009

@author: meloam
"""
from __future__ import print_function

try:
    # https://pylint.pycqa.org/en/latest/technical_reference/features.html
    # W1626: the `reload` built-in function is missing in python3
    # we can use imp.reload (deprecated) or importlib.reload
    from importlib import reload
except:
    pass

import copy
import logging
import os
import os.path
import shutil
import sys
import threading
import time
import unittest

from nose.plugins.attrib import attr

import WMCore.Storage.StageOutError as StageOutError
import WMCore.WMSpec.Steps.Builders.CMSSW as CMSSWBuilder
import WMCore.WMSpec.Steps.Builders.LogArchive as LogArchiveBuilder
import WMCore.WMSpec.Steps.Executors.LogArchive as LogArchiveExecutor
import WMCore.WMSpec.Steps.StepFactory as StepFactory
import WMCore.WMSpec.Steps.Templates.LogArchive as LogArchiveTemplate
import WMCore_t.WMSpec_t.samples.BasicProductionWorkload as testWorkloads
from WMCore.DataStructs.Job import Job
from WMCore.FwkJobReport.Report import Report
from WMCore.WMSpec.Makers.TaskMaker import TaskMaker
from WMCore.WMSpec.WMWorkload import newWorkload
from WMQuality.TestInit import TestInit


class LogArchiveTest(unittest.TestCase):
    def setUp(self):
        self.testInit = TestInit(__file__)
        self.testDir = self.testInit.generateWorkDir()

        # shut up SiteLocalConfig
        os.environ['CMS_PATH'] = os.getcwd()
        workload = copy.deepcopy(testWorkloads.workload)
        task = workload.getTask("Production")
        step = task.getStep("stageOut1")
        # want to get the cmsstep so I can make the Report
        cmsstep = task.getStep('cmsRun1')
        self.cmsstepdir = os.path.join(self.testDir, 'cmsRun1')
        os.mkdir(self.cmsstepdir)
        open(os.path.join(self.cmsstepdir, '__init__.py'), 'w').close()
        open(os.path.join(self.cmsstepdir, 'Report.pkl'), 'w').close()

        cmsbuilder = CMSSWBuilder.CMSSW()
        cmsbuilder(cmsstep.data, 'Production', self.cmsstepdir)
        realstep = LogArchiveTemplate.LogArchiveStepHelper(step.data)
        realstep.disableRetries()
        self.realstep = realstep
        self.stepDir = os.path.join(self.testDir, 'stepdir')
        os.mkdir(self.stepDir)
        builder = LogArchiveBuilder.LogArchive()
        builder(step.data, 'Production', self.stepDir)

        # stolen from CMSSWExecutor_t. thanks, dave

        # first, delete all the sandboxen and taskspaces
        #    because of caching, this leaks from other tests in other files
        #    this sucks because the other tests are using sandboxen that
        #    are deleted after the test is over, which causes theses tests
        #    to break
        modsToDelete = []
        # not sure what happens if you delete from
        # an arrey you're iterating over. doing it in
        # two steps
        for modname in sys.modules:
            # need to blow away things in sys.modules, otherwise
            # they are cached and we look at old taskspaces
            if modname.startswith('WMTaskSpace'):
                modsToDelete.append(modname)
            if modname.startswith('WMSandbox'):
                modsToDelete.append(modname)
        for modname in modsToDelete:
            try:
                reload(sys.modules[modname])
            except Exception:
                pass
            del sys.modules[modname]

        self.oldpath = sys.path[:]
        self.testInit = TestInit(__file__)

        self.testDir = self.testInit.generateWorkDir()
        self.job = Job(name="/UnitTests/DeleterTask/DeleteTest-test-job")
        shutil.copyfile('/etc/hosts', os.path.join(self.testDir, 'testfile'))

        self.workload = newWorkload("UnitTests")
        self.task = self.workload.newTask("DeleterTask")

        cmsswHelper = self.task.makeStep("cmsRun1")
        cmsswHelper.setStepType('CMSSW')
        stepHelper = cmsswHelper.addStep("DeleteTest")
        stepHelper.setStepType('LogArchive')

        self.cmsswstep = cmsswHelper.data
        self.cmsswHelper = cmsswHelper

        self.stepdata = stepHelper.data
        self.stephelp = LogArchiveTemplate.LogArchiveStepHelper(stepHelper.data)
        self.task.applyTemplates()

        self.executor = StepFactory.getStepExecutor(self.stephelp.stepType())
        taskMaker = TaskMaker(self.workload, os.path.join(self.testDir))
        taskMaker.skipSubscription = True
        taskMaker.processWorkload()

        self.task.build(os.path.join(self.testDir, 'UnitTests'))

        sys.path.insert(0, self.testDir)
        sys.path.insert(0, os.path.join(self.testDir, 'UnitTests'))

        #        binDir = inspect.getsourcefile(ModuleLocator)
        #        binDir = binDir.replace("__init__.py", "bin")
        #
        #        if not binDir in os.environ['PATH']:
        #            os.environ['PATH'] = "%s:%s" % (os.environ['PATH'], binDir)
        open(os.path.join(self.testDir, 'UnitTests', '__init__.py'), 'w').close()
        shutil.copyfile(os.path.join(os.path.dirname(__file__), 'MergeSuccess.pkl'),
                        os.path.join(self.testDir, 'UnitTests', 'WMTaskSpace', 'cmsRun1', 'Report.pkl'))

    def tearDown(self):
        sys.path = self.oldpath[:]
        self.testInit.delWorkDir()

        # making double sure WMTaskSpace and WMSandbox are gone
        modsToDelete = []
        # not sure what happens if you delete from
        # an arrey you're iterating over. doing it in
        # two steps
        for modname in sys.modules:
            # need to blow away things in sys.modules, otherwise
            # they are cached and we look at old taskspaces
            if modname.startswith('WMTaskSpace'):
                modsToDelete.append(modname)
            if modname.startswith('WMSandbox'):
                modsToDelete.append(modname)
        for modname in modsToDelete:
            try:
                reload(sys.modules[modname])
            except Exception:
                pass
            del sys.modules[modname]
        myThread = threading.currentThread()
        if hasattr(myThread, "factory"):
            myThread.factory = {}

    def makeReport(self, fileName):
        myReport = Report('oneitem')
        myReport.addStep('stageOut1')
        myReport.addOutputModule('module1')
        myReport.addOutputModule('module2')
        myReport.addOutputFile('module1', {'lfn': 'FILE1', 'size': 1, 'events': 1})
        myReport.addOutputFile('module2', {'lfn': 'FILE2', 'size': 1, 'events': 1})
        myReport.addOutputFile('module2', {'lfn': 'FILE3', 'size': 1, 'events': 1})
        myReport.persist(fileName)

    def testExecutorDoesntDetonate(self):
        myReport = Report()
        myReport.unpersist(os.path.join(self.testDir, 'UnitTests', 'WMTaskSpace', 'cmsRun1', 'Report.pkl'))
        myReport.data.cmsRun1.status = 1
        myReport.persist(os.path.join(self.testDir, 'UnitTests', 'WMTaskSpace', 'cmsRun1', 'Report.pkl'))

        executor = LogArchiveExecutor.LogArchive()

        executor.initialise(self.stepdata, self.job)
        self.setLocalOverride(self.stepdata)
        executor.step = self.stepdata
        executor.execute()
        self.assertFalse(os.path.exists(os.path.join(self.testDir, 'hosts')))
        self.assertFalse(os.path.exists(os.path.join(self.testDir, 'test1', 'hosts')))
        return

    def testUnitTestBackend(self):
        myReport = Report()
        myReport.unpersist(os.path.join(self.testDir, 'UnitTests', 'WMTaskSpace', 'cmsRun1', 'Report.pkl'))
        myReport.data.cmsRun1.status = 1
        myReport.persist(os.path.join(self.testDir, 'UnitTests', 'WMTaskSpace', 'cmsRun1', 'Report.pkl'))

        executor = LogArchiveExecutor.LogArchive()
        helper = LogArchiveTemplate.LogArchiveStepHelper(self.stepdata)
        helper.addOverride(override='command', overrideValue='test-win')
        helper.addOverride(override='option', overrideValue='')
        helper.addOverride(override='phedex-node', overrideValue='charlie.sheen.biz')
        helper.addOverride(override='lfn-prefix', overrideValue='test-win')

        executor.initialise(self.stepdata, self.job)
        self.setLocalOverride(self.stepdata)
        executor.step = self.stepdata
        executor.execute()
        self.assertFalse(os.path.exists(os.path.join(self.testDir, 'hosts')))
        self.assertFalse(os.path.exists(os.path.join(self.testDir, 'test1', 'hosts')))

    def testUnitTestBackendNew(self):
        myReport = Report()
        myReport.unpersist(os.path.join(self.testDir, 'UnitTests', 'WMTaskSpace', 'cmsRun1', 'Report.pkl'))
        myReport.data.cmsRun1.status = 1
        myReport.persist(os.path.join(self.testDir, 'UnitTests', 'WMTaskSpace', 'cmsRun1', 'Report.pkl'))

        executor = LogArchiveExecutor.LogArchive()
        helper = LogArchiveTemplate.LogArchiveStepHelper(self.stepdata)
        helper.addOverride(override='command', overrideValue='test-win')
        helper.addOverride(override='option', overrideValue='')
        helper.addOverride(override='phedex-node', overrideValue='charlie.sheen.biz')
        helper.addOverride(override='lfn-prefix', overrideValue='test-win')
        helper.setNewStageoutOverride(True)

        executor.initialise(self.stepdata, self.job)
        self.setLocalOverride(self.stepdata)
        executor.step = self.stepdata
        executor.execute()
        self.assertFalse(os.path.exists(os.path.join(self.testDir, 'hosts')))
        self.assertFalse(os.path.exists(os.path.join(self.testDir, 'test1', 'hosts')))

    def setLocalOverride(self, step):
        step.section_('override')
        step.override.command = 'cp'
        step.override.option = ''
        step.override.__setattr__('lfn-prefix', self.testDir + "/")
        step.override.__setattr__('phedex-node', 'DUMMYPNN')


class otherLogArchiveTexst(unittest.TestCase):
    def setUp(self):
        # stolen from CMSSWExecutor_t. thanks, dave

        # first, delete all the sandboxen and taskspaces
        #    because of caching, this leaks from other tests in other files
        #    this sucks because the other tests are using sandboxen that
        #    are deleted after the test is over, which causes theses tests
        #    to break
        modsToDelete = []
        # not sure what happens if you delete from
        # an arrey you're iterating over. doing it in
        # two steps
        for modname in sys.modules:
            # need to blow away things in sys.modules, otherwise
            # they are cached and we look at old taskspaces
            if modname.startswith('WMTaskSpace'):
                modsToDelete.append(modname)
            if modname.startswith('WMSandbox'):
                modsToDelete.append(modname)
        for modname in modsToDelete:
            try:
                reload(sys.modules[modname])
            except Exception:
                pass
            del sys.modules[modname]

        self.oldpath = sys.path[:]
        self.testInit = TestInit(__file__)

        self.testDir = self.testInit.generateWorkDir()
        self.job = Job(name="/UnitTests/DeleterTask/DeleteTest-test-job")
        shutil.copyfile('/etc/hosts', os.path.join(self.testDir, 'testfile'))

        self.workload = newWorkload("UnitTests")
        self.task = self.workload.newTask("DeleterTask")

        cmsswHelper = self.task.makeStep("cmsRun1")
        cmsswHelper.setStepType('CMSSW')
        stepHelper = cmsswHelper.addStep("DeleteTest")
        stepHelper.setStepType('LogArchive')

        self.cmsswstep = cmsswHelper.data
        self.cmsswHelper = cmsswHelper

        self.stepdata = stepHelper.data
        self.stephelp = LogArchiveTemplate.LogArchiveStepHelper(stepHelper.data)
        self.task.applyTemplates()

        self.executor = StepFactory.getStepExecutor(self.stephelp.stepType())
        taskMaker = TaskMaker(self.workload, os.path.join(self.testDir))
        taskMaker.skipSubscription = True
        taskMaker.processWorkload()

        self.task.build(os.path.join(self.testDir, 'UnitTests'))

        sys.path.insert(0, self.testDir)
        sys.path.insert(0, os.path.join(self.testDir, 'UnitTests'))

        #        binDir = inspect.getsourcefile(ModuleLocator)
        #        binDir = binDir.replace("__init__.py", "bin")
        #
        #        if not binDir in os.environ['PATH']:
        #            os.environ['PATH'] = "%s:%s" % (os.environ['PATH'], binDir)
        open(os.path.join(self.testDir, 'UnitTests', '__init__.py'), 'w').close()
        shutil.copyfile(os.path.join(os.path.dirname(__file__), 'MergeSuccess.pkl'),
                        os.path.join(self.testDir, 'UnitTests', 'WMTaskSpace', 'cmsRun1', 'Report.pkl'))

    def tearDown(self):
        sys.path = self.oldpath[:]
        self.testInit.delWorkDir()

        # making double sure WMTaskSpace and WMSandbox are gone
        modsToDelete = []
        # not sure what happens if you delete from
        # an arrey you're iterating over. doing it in
        # two steps
        for modname in sys.modules:
            # need to blow away things in sys.modules, otherwise
            # they are cached and we look at old taskspaces
            if modname.startswith('WMTaskSpace'):
                modsToDelete.append(modname)
            if modname.startswith('WMSandbox'):
                modsToDelete.append(modname)
        for modname in modsToDelete:
            try:
                reload(sys.modules[modname])
            except Exception:
                pass
            del sys.modules[modname]
        myThread = threading.currentThread()
        if hasattr(myThread, "factory"):
            myThread.factory = {}

    @attr('integration')
    def testCPBackendLogArchiveAgainstReportNew(self):
        myReport = Report()
        myReport.unpersist(os.path.join(self.testDir, 'UnitTests', 'WMTaskSpace', 'cmsRun1', 'Report.pkl'))
        myReport.data.cmsRun1.status = 0
        myReport.persist(os.path.join(self.testDir, 'UnitTests', 'WMTaskSpace', 'cmsRun1', 'Report.pkl'))
        executor = LogArchiveExecutor.LogArchive()
        executor.initialise(self.stepdata, self.job)
        self.setLocalOverride(self.stepdata)
        self.stepdata.override.newLogArchive = True
        executor.step = self.stepdata
        executor.execute()
        self.assertTrue(os.path.exists(os.path.join(self.testDir, 'hosts')))
        self.assertTrue(os.path.exists(os.path.join(self.testDir, 'test1', 'hosts')))

    @attr('integration')
    def testCPBackendLogArchiveAgainstReportFailedStepNew(self):
        myReport = Report()
        myReport.unpersist(os.path.join(self.testDir, 'UnitTests', 'WMTaskSpace', 'cmsRun1', 'Report.pkl'))
        myReport.data.cmsRun1.status = 1
        myReport.persist(os.path.join(self.testDir, 'UnitTests', 'WMTaskSpace', 'cmsRun1', 'Report.pkl'))
        executor = LogArchiveExecutor.LogArchive()
        executor.initialise(self.stepdata, self.job)
        self.setLocalOverride(self.stepdata)
        self.stepdata.override.newLogArchive = True
        executor.step = self.stepdata
        executor.execute()
        self.assertFalse(os.path.exists(os.path.join(self.testDir, 'hosts')))
        self.assertFalse(os.path.exists(os.path.join(self.testDir, 'test1', 'hosts')))
        return

    @attr('integration')
    def testCPBackendLogArchiveAgainstReportOld(self):

        myReport = Report()
        myReport.unpersist(os.path.join(self.testDir, 'UnitTests', 'WMTaskSpace', 'cmsRun1', 'Report.pkl'))
        myReport.data.cmsRun1.status = 0
        myReport.persist(os.path.join(self.testDir, 'UnitTests', 'WMTaskSpace', 'cmsRun1', 'Report.pkl'))
        executor = LogArchiveExecutor.LogArchive()
        executor.initialise(self.stepdata, self.job)
        self.setLocalOverride(self.stepdata)
        executor.step = self.stepdata
        executor.execute()
        self.assertTrue(os.path.exists(os.path.join(self.testDir, 'hosts')))
        self.assertTrue(os.path.exists(os.path.join(self.testDir, 'test1', 'hosts')))
        return

    @attr('integration')
    def testCPBackendLogArchiveAgainstReportFailedStepOld(self):
        myReport = Report()
        myReport.unpersist(os.path.join(self.testDir, 'UnitTests', 'WMTaskSpace', 'cmsRun1', 'Report.pkl'))
        myReport.data.cmsRun1.status = 1
        myReport.persist(os.path.join(self.testDir, 'UnitTests', 'WMTaskSpace', 'cmsRun1', 'Report.pkl'))

        executor = LogArchiveExecutor.LogArchive()
        executor.initialise(self.stepdata, self.job)
        self.setLocalOverride(self.stepdata)
        executor.step = self.stepdata
        executor.execute()
        self.assertFalse(os.path.exists(os.path.join(self.testDir, 'hosts')))
        self.assertFalse(os.path.exists(os.path.join(self.testDir, 'test1', 'hosts')))
        return

    @attr('workerNodeTest')
    def testOnWorkerNodes(self):
        raise RuntimeError
        # Stage a file out, stage it back in, check it, delete it
        myReport = Report()
        myReport.unpersist(os.path.join(self.testDir, 'UnitTests', 'WMTaskSpace', 'cmsRun1', 'Report.pkl'))
        myReport.data.cmsRun1.status = 1
        del myReport.data.cmsRun1.output
        myReport.data.cmsRun1.section_('output')
        myReport.data.cmsRun1.output.section_('stagingTestOutput')
        myReport.data.cmsRun1.output.stagingTestOutput.section_('files')
        myReport.data.cmsRun1.output.stagingTestOutput.fileCount = 0
        targetFiles = ['/store/temp/WMAgent/storetest-%s' % time.time(),
                       '/store/unmerged/WMAgent/storetest-%s' % time.time()]

        for file in targetFiles:
            print("Adding file for LogArchive %s" % file)
            self.addLogArchiveFile(myReport, file)

        myReport.persist(os.path.join(self.testDir, 'UnitTests', 'WMTaskSpace', 'cmsRun1', 'Report.pkl'))
        executor = LogArchiveExecutor.LogArchive()

        executor.initialise(self.stepdata, self.job)
        executor.step = self.stepdata
        print("beginning stageout")
        executor.execute()
        print("stageout done")

        # pull in the report with the stage out info
        myReport = Report()
        myReport.unpersist(os.path.join(self.testDir, 'UnitTests', 'WMTaskSpace', 'cmsRun1', 'Report.pkl'))
        print("Got the stage out data back")
        print(myReport.data)

        # now, transfer them back
        # TODO make a stagein step in the task - Melo
        import WMCore.Storage.FileManager as FileManagerModule
        fileManager = FileManagerModule.FileManager(numberOfRetries=10, retryPauseTime=1)
        for file in targetFiles:
            print("Staging in %s" % file)

            fileManager.stageOut(fileToStage={'LFN': file,
                                              'PFN': '%s/%s' % (self.testDir, file)})
            self.assertTrue(os.path.exists('%s/%s' % (self.testDir, file)))
            # self.assertEqual(os.path.getsize('/etc/hosts', '%s/%s' % (self.testDir, file)))

        # now, should delete the files we made
        for file in targetFiles:
            print("deleting %s" % file)
            fileManager.deleteLFN(file)

        # try staging in again to make sure teh files are gone
        for file in targetFiles:
            print("Staging in (should fail) %s" % file)
            self.assertRaises(StageOutError,
                              FileManagerModule.FileManager.stageOut,
                              fileManager, fileToStage={'LFN': file,
                                                        'PFN': '%s/%s' % (self.testDir, file)})

        # need to make sure files didn't show up
        self.assertFalse(os.path.exists(os.path.join(self.testDir, 'hosts')))
        self.assertFalse(os.path.exists(os.path.join(self.testDir, 'test1', 'hosts')))

    def addLogArchiveFile(self, myReport, lfn):
        myId = myReport.data.cmsRun1.output.stagingTestOutput.fileCount
        mySection = myReport.data.cmsRun1.output.stagingTestOutput.section_('file%s' % myId)
        mySection.section_('runs')
        setattr(mySection.runs, '114475', [33])
        mySection.section_('branches')
        mySection.lfn = lfn
        mySection.dataset = {'applicationName': 'cmsRun', 'primaryDataset': 'Calo',
                             'processedDataset': 'Commissioning09-PromptReco-v8', 'dataTier': 'ALCARECO',
                             'applicationVersion': 'CMSSW_3_2_7'}
        mySection.module_label = 'ALCARECOStreamCombined'
        mySection.parents = []
        mySection.location = 'srm-cms.cern.ch'
        mySection.checksums = {'adler32': 'bbcf2215', 'cksum': '2297542074'}
        mySection.pfn = '/etc/hosts'
        mySection.events = 20000
        mySection.merged = False
        mySection.size = 37556367
        myReport.data.cmsRun1.output.stagingTestOutput.fileCount = myId + 1

    def setLocalOverride(self, step):
        step.section_('override')
        step.override.command = 'cp'
        step.override.option = ''
        step.override.__setattr__('lfn-prefix', self.testDir + "/")
        step.override.__setattr__('phedex-node', 'DUMMYPNN')


if __name__ == "__main__":
    # import sys;sys.argv = ['', 'Test.testName']
    logging.basicConfig(level=logging.DEBUG)
    unittest.main()



#!/usr/bin/env python
"""
_UpdateBlocks_

MySQL implementation of DBS3Buffer.UpdateBlocks
"""

from WMCore.Database.DBFormatter import DBFormatter

class UpdateBlocks(DBFormatter):
    sql  = """UPDATE dbsbuffer_block SET status = :status, create_time = :time, location =
                (SELECT dbl.id FROM dbsbuffer_location dbl WHERE dbl.pnn = :location)
                WHERE blockname = :block"""

    def execute(self, blocks, conn = None, transaction = False):
        bindVars = []

        for block in blocks:
            bindVars.append({"block": block.getName(), "location": block.getLocation(),
                             "status": block.status, "time": block.getStartTime()})
        self.dbi.processData(self.sql, bindVars, conn = conn,
                                 transaction = transaction)

        return



#!/usr/bin/env python
"""
_AddToFileset_

Oracle implementation of Files.AddFileToFileset
"""

from WMCore.WMBS.MySQL.Files.AddToFileset import AddToFileset as AddFileToFilesetMySQL

class AddToFileset(AddFileToFilesetMySQL):
    sql = """INSERT INTO wmbs_fileset_files (fileid, fileset, insert_time)
               SELECT wmbs_file_details.id, :fileset, :insert_time
                      FROM wmbs_file_details
               WHERE wmbs_file_details.lfn = :lfn
               AND NOT EXISTS (SELECT fileid FROM wmbs_fileset_files wff2
                               WHERE wff2.fileid = wmbs_file_details.id AND
                                     wff2.fileset = :fileset)"""

    sqlAvail = """INSERT INTO wmbs_sub_files_available (subscription, fileid)
                    SELECT wmbs_subscription.id AS subscription,
                           wmbs_file_details.id AS fileid FROM wmbs_subscription
                      INNER JOIN wmbs_file_details ON
                        wmbs_file_details.lfn = :lfn
                    WHERE wmbs_subscription.fileset = :fileset AND NOT EXISTS
                      (SELECT * FROM wmbs_sub_files_available
                       WHERE fileid = (SELECT id FROM wmbs_file_details
                                     WHERE lfn = :lfn) AND
                             subscription = (SELECT id FROM wmbs_subscription
                                             WHERE fileset = :fileset AND rownum = 1))"""



#!/usr/bin/env python
"""
_MarkFinishedSubscriptions_

Oracle implementation of Subscriptions.MarkFinishedSubscriptions

Created on Aug 30, 2012

@author: dballest
"""

from WMCore.WMBS.MySQL.Subscriptions.MarkFinishedSubscriptions import MarkFinishedSubscriptions as MySQLMarkFinishedSubscriptions

class MarkFinishedSubscriptions(MySQLMarkFinishedSubscriptions):
    pass



"""
__ListForSubmitter__

Oracle implementation of ListForSubmitter

Created on Apr 17, 2013

@author: dballest
"""

from WMCore.WMBS.MySQL.Workflow.ListForSubmitter import ListForSubmitter as MySQLListForSubmitter

class ListForSubmitter(MySQLListForSubmitter):
    pass



"""
Unit tests for Unified/Common.py module

Author: Valentin Kuznetsov <vkuznet [AT] gmail [DOT] com>
"""
from __future__ import division, print_function

import json
import unittest

from nose.plugins.attrib import attr

from WMCore.MicroService.Tools.Common import (dbsInfo, getEventsLumis, findParent,
                                              hasHTTPFailed, isRelVal)


class CommonTest(unittest.TestCase):
    """Unit test for Common module"""

    def setUp(self):
        self.dbsUrl = "https://cmsweb-prod.cern.ch/dbs/prod/global/DBSReader"
        self.datasets = ['/ZMM/Summer11-DESIGN42_V11_428_SLHC1-v1/GEN-SIM',
                         '/ZMM_14TeV/Summer12-DESIGN42_V17_SLHCTk-v1/GEN-SIM']
        self.child = ['/SingleElectron/Run2016B-18Apr2017_ver2-v1/AOD']

    @attr("integration")
    def testDbsInfo(self):
        "Test function for dbsInfo()"
        datasetBlocks, datasetSizes, datasetTransfers = dbsInfo(self.datasets, self.dbsUrl)
        blocks = [b for d in self.datasets for b in datasetBlocks[d]]
        self.assertEqual(2, len(datasetBlocks))
        self.assertEqual(5, len(blocks))
        expect = 6271126523 + 7840499449
        sizes = sum([datasetSizes[d] for d in self.datasets])
        self.assertEqual(expect, sizes)
        self.assertEqual(len(self.datasets), len(datasetTransfers))

    @attr("integration")
    def testGetEventsLumis(self):
        "Test function for getEventsLumis()"
        totEvts = totLumis = 0
        for dataset in self.datasets:
            nevts, nlumis = getEventsLumis(dataset, self.dbsUrl)
            print("dataset: {} with evts: {} and lumis: {}".format(dataset, nevts, nlumis))
            totEvts += nevts
            totLumis += nlumis
        expect = 7398 + 0
        self.assertEqual(expect, totEvts)
        expect = 16 + 0
        self.assertEqual(expect, totLumis)

    @attr("integration")
    def test_findParent(self):
        "Test function for findParent()"
        parents = findParent(self.child, self.dbsUrl)
        self.assertEqual(parents[self.child[0]], '/SingleElectron/Run2016B-v2/RAW')

    def testHasHTTPFailed(self):
        """Test the hasHTTPFailed method"""
        self.assertTrue(hasHTTPFailed({}))
        self.assertTrue(hasHTTPFailed({'data': '', 'code': 301, 'error': ''}))
        self.assertTrue(hasHTTPFailed({'data': '', 'code': 301, 'error': 'blah'}))
        self.assertTrue(hasHTTPFailed({'data': 1, 'code': 400, 'error': 'blah'}))
        self.assertTrue(hasHTTPFailed({'data': 1, 'code': 500, 'error': []}))
        self.assertTrue(hasHTTPFailed({'data': 1, 'code': 500, 'error': None}))

        self.assertFalse(hasHTTPFailed({'data': []}))
        self.assertFalse(hasHTTPFailed({'data': None}))
        self.assertFalse(hasHTTPFailed({'data': json.dumps([])}))
        self.assertFalse(hasHTTPFailed({'data': json.dumps(None)}))
        self.assertFalse(hasHTTPFailed({'data': ['blah']}))
        self.assertFalse(hasHTTPFailed({'data': 'blah'}))
        self.assertFalse(hasHTTPFailed({'data': json.dumps('blah')}))
        self.assertFalse(hasHTTPFailed({'data': 'blah', 'code': 200}))
        # result below should never happen, but let's test the status code
        self.assertFalse(hasHTTPFailed({'data': 1, 'code': 200, 'error': 'blah'}))

    def testIsRelVal(self):
        """
        Test the `isRelVal` method functionality
        """
        badSubReqType = ("ReDigi", "TaskChain", "")
        goodSubReqType = ("RelVal", "HIRelVal")
        for subType in badSubReqType:
            testReqDict = {"RequestType": "StepChain", "DbsUrl": "a_dbs_url", "SubRequestType": subType}
            self.assertFalse(isRelVal(testReqDict))

        for subType in goodSubReqType:
            testReqDict = {"RequestType": "StepChain", "DbsUrl": "a_dbs_url", "SubRequestType": subType}
            self.assertTrue(isRelVal(testReqDict))

if __name__ == '__main__':
    unittest.main()
